package game;

import java.util.ArrayList;

import com.jogamp.opengl.GL2;

import audio.Audio;

import graphics.GFX;
import graphics.Tex;
import calc.Calc;
import calc.V3D;
import mechanics.Entity;
import mechanics.Scene;

public class Gilda extends Entity implements TailParent, Enemy
	{
	static final int STAND = 0;
	static final int WALK = 1;
	static final int JUMP = 2;
	static final int RUN = 3;
	static final int DUCK = 4;
	static final int DEAD = 5;
	
	static final int IDLE = 0;
	static final int MARCH = 1;
	static final int FLY = 2;
	static final int ATTACK = 3;
	static final int DEFEND = 4;
	
	static Gilda me;
	
	static final double RADIUS = 19-3+13;
	
	double headImg = 0;
	double bodyImg = 0;
	double wingImg = 0;
	double headRot = 0;
	
	int anim = STAND;
	int mode = IDLE;
	
	double bwalk=0.18;
	double fwalk=0.32;
	int walkTime=20;
	double runPhase=0;
	int runTime=15;
	
	int blinkTimer=30+(int)(90*Math.random());
	
	double yPlus=0;
	
	static Tex sB1 = GFX.tex("PonyThigh");
	static Tex sB3 = GFX.tex("PonyRearHoof");
	
	static Tex sF1 = GFX.tex("PonyElbow");
	static Tex sF3 = GFX.tex("GildaClaw");
	static{
		sF3.origin.setLocation(2+5, 3);
		}
	
	final double walkAmplitude = 32;
	final double walkSpeed = walkAmplitude/walkTime*1.2;
	final double runSpeed = walkSpeed*3.2;
	
	double BRx=3+0.07*walkAmplitude+1;
	double BRy=0;
	double BRi=0;
	double BLx=3-0.07*walkAmplitude;
	double BLy=0;
	double BLi=0;

	double FRx=0.07*walkAmplitude+1;
	double FRy=0;
	double FRi=0;
	double FLx=-0.07*walkAmplitude;
	double FLy=0;
	double FLi=0;
	
	Tail firstTail;
	
	double targetX;
	double returnTargetX;
	double targetY;
	double yStart;
	/*Whether to hang head when standing or walking.*/
	private boolean headDown;
	private int life;
	private int lifeCheckpoint;
	private boolean attackSwitch = true;
	
	private int timeout;
	private int actionBeat;
	
	public Gilda(V3D pos)
		{
		super(pos);
		PataController.addEnemy(this);
		life = 60;
		lifeCheckpoint = 40;
		actionBeat = -1;
		yStart = pos.y();
		headDown=false;
		me = this;
		targetX = pos.x();
		targetY = -1;
		timeout = 120;
		
		Calc.fill(color, 0.75f, 0.63f, 0.34f, 1f);
		
		int xx=13, yy=24;
		GFX.tex("GildaHead").origin.setLocation(xx, yy);
		xx=0; yy=10;
		GFX.tex("GildaWing").origin.setLocation(xx, yy);
		
		tailRegen();
		}
	
	@Override
	public void destroy()
		{
		super.destroy();
		me = null;
		}
	
	@Override
	public void step()
		{
		if(Pinkie.flagHolder.pos.x()+192>returnTargetX)
			returnTargetX = Pinkie.flagHolder.pos.x()+192;
		if(mode!=ATTACK && mode!=JUMP)
			returnTargetX = targetX;
		
		if(timeout>0)
			timeout--;
		if(anim!=DEAD)
			{
			if(actionBeat!=-1 && PataController.me.currentBeat>actionBeat)
				{
				if(mode==FLY)
					{
					Audio.get("Roar").play();
					headImg = 3;
					alarm[0]=45;
					
					new Roar(pos.clone().add(-30,0,0));
					
					alarm[1]=5;
					}
				else if(mode==DEFEND)
					{
					headImg = 3;
					mode = ATTACK;
					targetX = Pinkie.flagHolder.pos.x()-128;
					}
				actionBeat = -1;
				}
			if(mode==MARCH)
				{
				if (Math.abs(targetX-pos.x())<runSpeed+1)
					{
					mode = IDLE;
					}
				}
			else if(mode==IDLE)
				{
				if(timeout<=0 && (PataController.me.currentBeat%4)+((PataController.me.userInputOdd ^ PataController.me.currentBeat%8<4)?0:4)==3)
					{
					if(life<lifeCheckpoint)
						{
						targetX+=320;
						lifeCheckpoint-=20;
						mode = MARCH;
						}
					else if(attackSwitch || returnTargetX > Pinkie.flagHolder.pos.x()+256)
						{
						speed.set(0,-4,0);
						pos.add(speed);
						anim = JUMP;
						mode = FLY;
						headImg=0;
						targetY = PataController.GND_Y-RADIUS-96;
						actionBeat = PataController.me.currentBeat+11;
						attackSwitch = false;
						}
					else
						{
						headImg=0;
						mode = DEFEND;
						targetX = getReturnX()+128;
						actionBeat = PataController.me.currentBeat+10;
						attackSwitch = true;
						}
					}
				}
			
			ArrayList<Pinkie> ps = Pinkie.pinkies;
			int i=0;
			while (i<ps.size())
				{
				Pinkie p = ps.get(i);
				if(pos.distance(p.pos)<Pinkie.RADIUS+RADIUS && p.anim!=Pinkie.DEAD)
					{
					int hit = 0;
					int panim = p.anim;
					if(panim!=JUMP)
						{
						if(anim==RUN)
							{
							hit++;
							p.hit((((panim==Pinkie.RUN || p.mode==Pinkie.ATTACK) && mode!=ATTACK) || p==Pinkie.flagHolder)?0:3);
							}
						else
							p.hit(0);
						
						if((panim==Pinkie.RUN || mode==ATTACK) && mode!=DEFEND)
							{
							hit++;
							hit(PataController.me.isFever()?2:1);
							}
						
						if(hit==0)
							Audio.get("blow1").play();
						else
							Audio.get("blow2").play();
						}
					}
				i++;
				}
			}
		
		if(anim==DEAD)
			{
			speed.p[1]+=0.25;
			if(pos.y()>PataController.GND_Y+32)
				{
				new Explosion(pos.clone().y(PataController.GND_Y));
				destroy();
				}
			}
		else
			{
			if(anim==JUMP)
				{
				if(targetY==-1)
					{wingImg = 0;
					speed.p[1]+=0.25;}
				else
					{
					wingImg += 0.2;
					if(wingImg>=3)
						wingImg -=2;
					targetX = Pinkie.flagHolder.pos.x()+224;
					if(pos.y()>targetY)
						speed.p[1]-=0.25;
					else
						speed.p[1]+=0.25;
					if(Math.abs(pos.p[1]-targetY)<4 && Math.abs(speed.p[1])<0.4)
						speed.p[1]=0;
					
					if(pos.x()>targetX)
						speed.p[0]-=0.2;
					if(pos.x()<targetX)
						speed.p[0]+=0.2;
					speed.p[0]/=1.075;
					speed.p[1]/=1.1;
					}
				if(pos.y()>=yStart)
					{
					speed.set(0,0,0);
					pos.y(yStart);
					anim = STAND;
					headImg=0;
					}
				}
			if(anim!=JUMP)
				{
				wingImg = 0;
				
				if(Math.abs(targetX-pos.x())>runSpeed)
					{
					/*if(mode==MARCH)
						{
						if(anim!=WALK)
							{bwalk=0.18;
							fwalk=0.32;
							
							BRx=3+0.07*walkAmplitude+1;
							BRy=0;
							BRi=0;
							BLx=3-0.07*walkAmplitude;
							BLy=0;
							BLi=0;
							
							FRx=0.07*walkAmplitude+1;
							FRy=0;
							FRi=0;
							FLx=-0.07*walkAmplitude;
							FLy=0;
							FLi=0;}
						anim = WALK;
						}
					else*/
						anim=RUN;
					
					if(pos.x()>targetX)	
						{
						setXScale(-1);
						pos.p[0]-=anim==RUN?runSpeed:walkSpeed;
						}
					else
						{
						setXScale(1);
						pos.p[0]+=anim==RUN?runSpeed:walkSpeed;
						}
					}
				else
					{
					if(mode==ATTACK)//Nothing there
						{
						headImg = 0;
						targetX = getReturnX();
						mode = IDLE;
						}
					
					if(mode==DEFEND)
						anim = DUCK;
					else
						anim = STAND;
					setXScale(-1);
					}
				}
			
			
			blinkTimer-=1;
			if(headImg<2)
				headImg = (blinkTimer<3?1:0);
			if(blinkTimer<=0)
				blinkTimer=30+(int)(90*Math.random());
			}
		anim();
		
		tailUpdate();
		super.step();
		}
	private double getReturnX()
		{
		return returnTargetX;
		}

	public void die()
		{
		new Explosion(pos.clone());
		speed.add(4,-4,0);
		anim = DEAD;
		headImg=2;
		}

	private void setXScale(int i)
		{
		boolean b = false;
		if(xScale!=i)
			b=true;
		xScale=i;
		if(b)
			tailRegen();
		}

	private void anim()
		{
		if(anim==DEAD)
			{
			bodyImg = 1;
			headImg = 2;
			headRot = (headRot+Calc.toRad(60))/3;
			angle+=Calc.toRad(9);
			
			bwalk=0.18;
			fwalk=0.32;
			double up = -6;
			yPlus = (yPlus*2-up)/3;
			
			up = -6;
			BRx=7+0.07*walkAmplitude+1-12;
			BRy=(BRy*2+up)/3;
			BRi=2;
			BLx=7-0.07*walkAmplitude-12;
			BLy=(BLy*2+up)/3;
			BLi=2;
			
			
			up = -14;
			FRx=0.07*walkAmplitude-1;
			FRy=(FRy*2+up+8)/3;
			FRi=3;
			FLx=-0.07*walkAmplitude;
			FLy=(FLy*2+up+8)/3;
			FLi=3;
			}
		else if(anim==DUCK)
			{
			bodyImg = 5;
			headImg = 2;
			headRot = (headRot+Calc.toRad(-90))/3;
			bwalk=0.18;
			fwalk=0.32;
			double up = -14;
			yPlus = (yPlus*2-up)/3;
			
			BRx=7+0.07*walkAmplitude+1;
			BRy=(BRy*2+up)/3;
			BRi=0;
			BLx=7-0.07*walkAmplitude;
			BLy=(BLy*2+up)/3;
			BLi=0;
			
			
			
			FRx=0.07*walkAmplitude-1;
			FRy=(FRy*2+up+8)/3;
			FRi=3;
			FLx=-0.07*walkAmplitude;
			FLy=(FLy*2+up+8)/3;
			FLi=3;
			}
		else if(anim==STAND)
			{
			yPlus=0;
			if(headDown)
				headRot=(headRot*2+Calc.toRad(-30))/3;
			else
				headRot=headRot/3;
			bwalk=0.18;
			fwalk=0.32;
			
			BRx=3+0.07*walkAmplitude+1;
			BRy=0;
			BRi=0;
			BLx=3-0.07*walkAmplitude;
			BLy=0;
			BLi=0;
			
			FRx=0.07*walkAmplitude+1;
			FRy=0;
			FRi=0;
			FLx=-0.07*walkAmplitude;
			FLy=0;
			FLi=0;
			
			bodyImg=0;
			}
		else if(anim==WALK)
			{
			yPlus=(PataController.beatTimer<15?1:0);
			if(headDown)
				headRot=(headRot*2+Calc.toRad(-20))/3;
			else
				headRot=headRot/3;
			int a=1;
			int h=3;
			bodyImg = (bwalk*4);
			bwalk+=1d/walkTime;
			if (bwalk>=1)
			    bwalk-=1;
			fwalk+=1d/walkTime;
			if (fwalk>=1)
			    fwalk-=1;
			
			int c;
			double rm;
			double rp;
			
			if(bwalk<0.5)
			    {
			    c=-1;
			    rm=-h;
			    rp=0;
			    BLi=1;
			    BRi=0;
			    }
			else
			    {
			    c=1;
			    rp=-h;
			    rm=0;
			    BLi=0;
			    BRi=1;
			    }
			BRx+=(c*walkAmplitude)/walkTime;
			BRy=(BRy*a+rp)/(a+1);
			BLx-=(c*walkAmplitude)/walkTime;
			BLy=(BLy*a+rm)/(a+1);
			if(fwalk<0.5)
			    {
			    c=-1;
			    rm=-h;
			    rp=0;
			    FLi=0;
			    FRi=1;
			    }
			else
			    {
			    c=1;
			    rp=-h;
			    rm=0;
			    FLi=1;
			    FRi=0;
			    }
			FRx-=c*walkAmplitude/walkTime;
			FRy=(FRy*a+rm)/(a+1);
			FLx+=c*walkAmplitude/walkTime;
			FLy=(FLy*a+rp)/(a+1);
			}
		else if (anim==JUMP)
			{
				
			yPlus=0;
			bwalk=0.18;
			fwalk=0.32;
			if(speed.p[1]<=-0.1)
			    {
			    BRx=(BRx*1+3+0.07*walkAmplitude+1)/2;
			    BRy=(BRy*1+2)/2;
			    BLx=(BLx*1+3)/2;
			    BLy=(BLy*1+3)/2;
			    FRx=(FRx*1+8)/2;
			    FRy=(FRy*1+-5)/2;
			    FLx=(FLx*1+-0.07*walkAmplitude)/2;
			    FLy=(FLy*1+-3)/2;
			    headRot=(headRot*1+Calc.toRad(10))/2;
			    }
			else
			    {
			    headRot=(headRot*2-Calc.toRad(30))/3;
			    BRx=(BRx*2+8)/3;
			    BRy=(BRy*2+-4)/3;
			    BLx=(BLx*2+3)/3;
			    BLy=(BLy*2+1)/3;
			    FRx=(FRx*2+8)/3;
			    FRy=(FRy*2+1)/3;
			    FLx=(FLx*2+-0.07*walkAmplitude)/3;
			    FLy=(FLy*2+0)/3;
			    }
			BRi=(speed.p[1]<0?1:0);
			BLi=(speed.p[1]<0?1:0);
			FRi=(speed.p[1]<0?1:0);
			FLi=(speed.p[1]<0?1:0);
			}
		else if(anim==RUN)
			{
			double t;
			yPlus=0;
			
			
			headRot=(headRot*2+Calc.toRad(runPhase>0.5?(-35):(-22)))/3;
			runPhase += 1d/runTime;
			runPhase = runPhase%1;
			bodyImg=((runPhase>0.25 && runPhase<0.75)?4:5);
			
			BLi = (runPhase>0.25 && runPhase<0.75)?0:2;
			if(runPhase<0.5)
				BLx = walkAmplitude*1.5*(0.03+runPhase-0.25);
			else
				BLx = walkAmplitude*1.5*(0.03+0.75-runPhase);
			t = (runPhase>0.5 && runPhase<0.75)?0:-3;
			BLy = (BLy+t)/2;
			
			double arunPhase = runPhase-0.15;
			if(arunPhase<0)
				arunPhase+=1;
			double brunPhase = runPhase-0.3;
			if(brunPhase<0)
				brunPhase+=1;
			BRi = (arunPhase>0.25 && arunPhase<0.75)?0:2;
			if(arunPhase<0.5)
				BRx = walkAmplitude*1.5*(0.03+arunPhase-0.25);
			else
				BRx = walkAmplitude*1.5*(0.03+0.75-arunPhase);
			t = (arunPhase>0.5 && arunPhase<0.75)?0:-3;
			BRy = (BRy+t)/2;
			
			FRi = (arunPhase>0.25 && arunPhase<0.75)?0:2;
			if(arunPhase<0.5)
				FRx = walkAmplitude*1.5*(-0.01+arunPhase-0.25);
			else
				FRx = walkAmplitude*1.5*(-0.01+0.75-arunPhase);
			t = (arunPhase>0.5 && arunPhase<0.75)?0:-3;
			FRy = (FRy+t)/2;
			
			FLi = (brunPhase>0.25 && brunPhase<0.75)?0:2;
			if(brunPhase<0.5)
				FLx = walkAmplitude*1.5*(-0.01+brunPhase-0.25);
			else
				FLx = walkAmplitude*1.5*(-0.01+0.75-brunPhase);
			t = (brunPhase>0.5 && brunPhase<0.75)?0:-3;
			FLy = (FLy+t)/2;
			}
		}

	public void hit(int damage)
		{
		if(anim!=JUMP && anim!=DEAD)
			{
			if(damage>0 && anim!=DEFEND)
				{
				life -= damage;
				Audio.get("GildaHurt"+String.valueOf(1+(int)(2*Math.random()))).play();
				}
			speed.set(2,-3,0);
			pos.add(speed);
			anim = JUMP;
			headImg = 2;
			targetX = getReturnX();
			if(life<=0)
				die();
			}
		}
	
	private void tailUpdate()
		{
		double h=Math.cos(angle);
		double v=-Math.sin(angle);
		double h2=-v;
		double v2=h;

		h*=xScale;
		v*=xScale;

		firstTail.pos.p[0]=pos.x()+(-12+3)*h+(-15+13+yPlus)*h2;
		firstTail.pos.p[1]=pos.y()+(-12+3)*v+(-15+13+yPlus)*v2;
		firstTail.x = new V3D(  -h*1.2-h2*1.2,
						 		-v*1.2-v2*1.2,
						 		0);
		firstTail.speed.set(speed);
		firstTail.speed.p[0]+=firstTail.x.x();
		firstTail.speed.p[1]+=firstTail.x.y();
		if(firstTail.next!=null)
			{
			firstTail.next.speed.add(firstTail.x,1.2);
			firstTail.next.next.speed.add(firstTail.x,0.8);
			firstTail.next.next.next.speed.add(firstTail.x,0.3);
			}
		}
	private void tailRegen()
		{
		Entity[] es = Scene.getInstances();
		for(Entity e:es)
			{
			if(e instanceof Tail)
				{
				if(((Tail) e).parent==this)
					e.destroy();
				}
			}
		int depth=-1;
		firstTail = new Tail(new V3D(0,0,0),color,true,this,0,9);
		firstTail.setDepth(depth);
		tailUpdate();
		Tail p = firstTail;
		for(int n=1; n<=8; n++)
		    {
		    depth--;
		    Tail  i= new Tail(new V3D(p.pos.x()-1*xScale,p.pos.y()+2,0),color,false,this,n,9);
		    firstTail.setDepth(depth);
		    p.next=i;
		    i.prev=p;
		    p=i;
		    }
		}
	@Override
	public void render()
		{
		GL2 gl = GFX.gl;
		gl.glPushMatrix();
		
		gl.glPushMatrix();
		gl.glTranslated(pos.x(), pos.y(), pos.z());
		gl.glRotated(Calc.toDeg(angle), 0, 0, -1);
		gl.glScaled(xScale,1,1);
		gl.glTranslated(-12,-15+yPlus,0);
		
		renderLegs(-1,0,false,0.5f);
		renderLegs(1,0,false,0.5f);
		
		renderLegs(-1,0,true,0.5f);
		renderLegs(1,0,true,0.5f);

		gl.glColor4fv(color,0);
		GFX.drawTexture(GFX.tex("PonyBody"), (int)bodyImg, 0, 0, 0, GFX.NORTHWEST, 1, 1, 0);
		renderLegs(0,0,false,1f);
		
		gl.glColor4fv(GFX.WHITE,0);
		GFX.drawTexture(GFX.tex("GildaHead"), (int)headImg, (bodyImg<4?13:24)+9, (bodyImg<4?-17:-9)+17, 0, GFX.NORTHWEST, 1f, 1f, (float)(headRot));
		
		gl.glColor4fv(color,0);
		GFX.drawTexture(GFX.tex("GildaWing"), (int)wingImg, 0, 0, 0, GFX.NORTHWEST, 1, 1, 0);
		
		gl.glPopMatrix();
		
		gl.glColor4fv(color,0);
		Tex gildaTail = GFX.tex("GildaTail");
		gildaTail.enable(0);
		
		gl.glBegin(GL2.GL_TRIANGLE_STRIP);
		Tail curr = firstTail;
		double tailRadius = 7;
		for(int i=0; i<=8; i++)
			{
			float cx = 1-(float)i/8;
			double h = tailRadius*Math.cos(curr.dir+Math.PI/2);
			double v = -tailRadius*Math.sin(curr.dir+Math.PI/2);
			gl.glTexCoord2f(gildaTail.xMult*cx, gildaTail.yMult*(0.5f+(float)xScale/2));
			gl.glVertex2d(curr.pos.x()+h, curr.pos.y()+v);
			gl.glTexCoord2f(gildaTail.xMult*cx, gildaTail.yMult*(0.5f-(float)xScale/2));
			gl.glVertex2d(curr.pos.x()-h, curr.pos.y()-v);
			curr = curr.next;
			}
		gl.glEnd();
		gl.glDisable(GL2.GL_TEXTURE_2D);
		
		gl.glPopMatrix();
		}
	/**
	 * 
	 * @param x offset
	 * @param y offset
	 * @param f color multiplier
	 */
	private void renderLegs(int x, int y, boolean right, float f)
		{
		final double bx=-8;
		final double fx=9;
		final double dy=13;
		
		final double ox=12;
		final double oy=15;
		
		GFX.gl.glColor3f(color[0]*f,color[1]*f,color[2]*f);
		GL2 gl = GFX.gl;
		if(!right)
			{
			//L
			gl.glPushMatrix();
			gl.glTranslated(x,y,0);
			gl.glPushMatrix();
			
			gl.glTranslated(ox+bx,oy,0);
			GFX.drawTexture(sB1,0,0,0,0,GFX.NORTHWEST,1,1,(float) Calc.dir(0,0,BLx,BLy+dy));
			gl.glTranslated(BLx,BLy+dy,0);
			GFX.drawTexture(sB3,(int) BLi,0,0,0,GFX.NORTHWEST,1,1,0);
	
			gl.glPopMatrix();
			
			gl.glTranslated(ox+fx,oy,0);
			GFX.drawTexture(sF1,0,0,0,0,GFX.NORTHWEST,1,1,(float) Calc.dir(0,0,FLx,FLy+dy));
			gl.glTranslated(FLx,FLy+dy,0);
			GFX.gl.glColor3f(f,f,f);
			GFX.drawTexture(sF3,(int) FLi,0,0,0,GFX.NORTHWEST,1,1,0);
	
			gl.glPopMatrix();
			}
		else
			{
			//R
			gl.glPushMatrix();
			gl.glTranslated(x,y,0);
			gl.glPushMatrix();
	
			gl.glTranslated(ox+bx,oy,0);
			GFX.drawTexture(sB1,0,0,0,0,GFX.NORTHWEST,1,1,(float) Calc.dir(0,0,BRx,BRy+dy));
			gl.glTranslated(BRx,BRy+dy,0);
			GFX.drawTexture(sB3,(int) BRi,0,0,0,GFX.NORTHWEST,1,1,0);
	
			gl.glPopMatrix();
			
			gl.glTranslated(ox+fx,oy,0);
			GFX.drawTexture(sF1,0,0,0,0,GFX.NORTHWEST,1,1,(float) Calc.dir(0,0,FRx,FRy+dy));
			gl.glTranslated(FRx,FRy+dy,0);
			GFX.gl.glColor3f(f,f,f);
			GFX.drawTexture(sF3,(int) FRi,0,0,0,GFX.NORTHWEST,1,1,0);
			
			gl.glPopMatrix();
			}
		}

	@Override 
	public void alarm(int n)
		{
		if(n==0)//Stop flying
			{
			targetY = -1;
			speed.set(4,0,0);
			headImg=0;
			mode = IDLE;
			timeout = 180;
			targetX = returnTargetX;
			alarm[1]=0;
			}
		else if (n==1)
			{
			new Roar(pos.clone().add(-30,0,0));
			alarm[1]=5;
			}
		/*else if(n==1)//Charge
			{
			eyeImg = 2;
			headImg = 2;
			mode = ATTACK;
			targetX = flagHolder.targetX+(PataController.me.numValidMeasures>=10?320:160)+Math.random()*64;
			}
		else if(n==2)//Duck
			{
			mode = DEFEND;
			targetX = pos.x();
			}
		else if(n==3)//Unknown
			{
			}
		else if(n==4)//Fail
			{
			if(Math.random()<0.5)
				eyeImg = 2;
			if(Math.random()<0.5)
				headDown = true;
			headImg = 2;
			alarm[6]=30;
			
			}
		else if(n==5)//Stop flagging(flag holder only)
			{
			eyeImg = 0;
			headImg = 0;	
			setFlagType(IDLE);
			}
		else */
		if(n==6)//Reset headEyeImg
			{
			headImg = 0;
			headDown = false;
			}
		}

	@Override
	public V3D getPosition()
		{
		return pos;
		}

	@Override
	public double getXScale()
		{
		return xScale;
		}

	@Override
	public boolean isTwitching()
		{
		return false;
		}	
	}
