package game;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import io.IOH;
import graphics.BlendMode;
import graphics.GFX;

import com.jogamp.opengl.GL2;

import audio.Audio;
import audio.JavaSoundListener;
import audio.StupidClip;

import calc.Calc;
import calc.V3D;
import main.Main;
import mechanics.Entity;

/**Monolithic main menu class*/
public class Menu extends Entity implements AnimationListener, JavaSoundListener, KeyListener
{
	PonyAnimator pony;
	int count = 0;
	float scroll = 0;

	static final String[] name = {"BASICS","BASICS(no intro)","PARASPRITES","GILDA GRIFFON","END CREDITS","QUIT"};
	static final int BUTTONS = 6;
	/**This is the index of the last selectable button.*/
	private static int progress = 0;
	/**Set the index of the last selectable button. This will write to file.*/
	public static void setProgress(int prog)
	{
		if(prog>progress)
		{
			progress = prog;
			BufferedWriter b = null;
			try
			{
				b = new BufferedWriter(new FileWriter("progress.dat"));
				b.write(String.valueOf(name[progress]));
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
			finally
			{
				if(b != null) try {b.close();} catch(IOException err) {err.printStackTrace();}
			}
		}
	}
	/**Look for the progress file and get the progress.*/
	public static void readProgress()
	{
		File f = new File("progress.dat");
		if(f.exists())
		{
			BufferedReader b = null;
			try
			{
				b = new BufferedReader(new FileReader("progress.dat"));
				String s = b.readLine();
				for(int i=0; i<name.length; i++)
				{
					if(s.equals(name[i]))
					{
						progress = i;
						break;
					}
				}
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
			finally
			{
				if(b != null) try {b.close();} catch(IOException err) {err.printStackTrace();}
			}
		}
	}
	/**Current selected button index*/
	int selected = 0;
	/**Whether we are starting. -1 for no, the index otherwise*/
	private int start;
	private boolean redrawn;


	public Menu(V3D pos)
	{
		super(pos);
		start = -1;
		redrawn = false;
		alarm[0]=2;
		PataController.setMusicName("CC");
		Audio.addJavaSoundListener(this);

		GFX.gl.glEnable(GL2.GL_BLEND);
		BlendMode.NORMAL.set();

		pony = new PonyAnimator(new V3D(320-64,0,0),true);
		pony.scale = 40;
		pony.pos.p[1]=240-1.509*pony.scale;
		pony.animate(PonyAnimator.DANCEBEGIN);
		pony.animate = false;
		pony.setAnimationListener(this);
		pony.mirror = true;
		IOH.addKeyListener(this);
		Calc.matrixMultiply3OptFirst(pony.master.postMatrix, Calc.makeZMatrix((float) (-Math.PI*2/6)));
	}
	@Override
	public void destroy()
	{
		super.destroy();
		Audio.removeJavaSoundListener(this);
	}

	@Override
	public void alarm(int index)
	{
		if(index==0)
		{
			StupidClip c = Audio.get("CC-0");
			c.play();
			c.setPositionListener(this, c.getFrameRate());
		}
	}

	@Override
	public void step()
	{
		super.step();
		scroll+=128f/60;
		if(scroll>128)
			scroll=0;

		if(start!=-1 && redrawn)
		{
			Audio.get("CC-0").stop();
			Audio.get("CC-1").stop();
			destroy();
			pony.destroy();
			if(selected<2)
				PataController.levelIndex = 0;
			else
				PataController.levelIndex = selected-1;

			if(selected==1)
				PataController.skipIntro = true;
			else
				PataController.skipIntro = false;
			new PataController(V3D.ZERO.clone());
		}
	}
	@Override
	public void render()
	{
		GL2 gl = GFX.gl;

		gl.glClearColor(0,0,0,1);
		gl.glClear(GL2.GL_COLOR_BUFFER_BIT);

		gl.glLoadIdentity();
		gl.glOrtho(0,320,240,0, -6400, 6400);

		if(start==-1)
		{
			gl.glColor3f(1,1,1);
			gl.glBegin(GL2.GL_TRIANGLE_STRIP);
			gl.glColor3f(1,0.5f,0);
			gl.glVertex2f(160, 0);
			gl.glVertex2f(320, 0);
			gl.glColor3f(0.95f,0.77f,0.3f);
			gl.glVertex2f(160, 240);
			gl.glVertex2f(320, 240);
			gl.glColor3f(1,1,1);
			gl.glEnd();

			gl.glColor4f(1,1,1,0.33f);
			int x = (int) (160+scroll-128);
			int y = (int) (scroll-128);
			for(int i=0; i<3; i++){
				for(int j=0; j<3; j++){
					GFX.drawTexture(GFX.tex("orange"), 0, x+i*128, y+j*128, 0, GFX.NORTHWEST, 1, 1, 0);}}
			gl.glColor3f(1,1,1);
			GFX.drawTexture(GFX.tex("PataPony"), 0, 160, 0, 0, GFX.NORTHWEST, 1, 1, 0);

			for(int i=0; i<BUTTONS; i++)
			{
				if(i==selected)
					gl.glColor3f(1,1,1);
				else if(i<BUTTONS-1 && i>progress)
					gl.glColor3f(0,0,0);
				else
					gl.glColor3f(0.2f,0.2f,0.2f);

				GFX.drawTexture(GFX.tex("buttonFrame"), 0, 0, i*40, 0, GFX.NORTHWEST, 1, 1, 0);
			}
			gl.glColor3f(1,1,1);
			x = 8;

			Overlay.textBegin(Overlay.arialBlack16);
			for(int i=0; i<BUTTONS; i++)
			{
				y = i*40 + 8;
				String str;
				if(i==progress && i!=0 && i!=BUTTONS-1)
					str = "???";
				else if(i<BUTTONS-1 && i>progress)
					str = null;
				else
					str = name[i];

				if(str!=null)
				{
					if(i==selected)
						Overlay.textDraw(Overlay.arialBlack16, x, y, str, 1);
					else
						Overlay.textDraw(Overlay.arialBlack16, x, y, str, 0.5f);
				}
			}
			Overlay.textEnd(Overlay.arialBlack16);
		}
		else
		{
			pony.visible = false;
			redrawn = true;
		}
	}

	@Override
	public void ponyAnimatorAnimationComplete(int animationIndex)
	{
		int i = 0;
		if(count>16)
			count = 0;

		if(count < 4)
			i = PonyAnimator.DANCEJUMP;
		else if(count ==4)
			i = PonyAnimator.UPSTART;
		else if(count < 7)
			i = PonyAnimator.UPLOOP;
		else if(count ==7)
			i = PonyAnimator.UPEND;
		else if(count < 12)
			i = PonyAnimator.DANCEJUMP;
		else if(count == 12)
			i = PonyAnimator.BUTTSTART;
		else if(count < 16)
			i = PonyAnimator.BUTTLOOP;
		else if(count == 16)
			i = PonyAnimator.BUTTEND;

		count++;
		pony.animate(i);
	}

	@Override
	public void soundStop(StupidClip c, int milliseconds)
	{
		if(start==-1)
		{
			if(c.getName().equals("CC-0") || c.getName().equals("CC-1"))
			{
				StupidClip cc = Audio.get("CC-1");
				cc.setPositionListener(this, c.getFrameRate());
				cc.play();
			}
		}
	}

	@Override
	public void soundAction(StupidClip c, int framePosition)
	{
		if(start==-1)
		{
			if(c.getName().equals("CC-1"))
			{
				pony.frame = 760+30*(int)Math.round((pony.frame-760)/30);
				c.setPositionListener(this, framePosition+c.getFrameRate());
			}
			else if(c.getName().equals("CC-0"))
			{
				pony.animate = true;
			}
		}
	}
	@Override
	public void keyPressed(KeyEvent e)
	{
		if(start==-1)
		{
			if(e.getKeyCode()==KeyEvent.VK_DOWN)
			{
				Audio.get("beep").play();
				selected++;
				if(selected>=BUTTONS)
					selected=0;
				else if(selected>progress)
					selected = BUTTONS-1;
			}
			else if(e.getKeyCode()==KeyEvent.VK_UP)
			{
				Audio.get("beep").play();
				selected--;
				if(selected<0)
					selected=BUTTONS-1;
				else if(selected>progress)
					selected = progress;
			}
			else if(e.getKeyCode()==KeyEvent.VK_ENTER || e.getKeyCode()==KeyEvent.VK_SPACE ||
					e.getKeyCode()==KeyEvent.VK_A || e.getKeyCode()==KeyEvent.VK_S || e.getKeyCode()==KeyEvent.VK_D)
			{
				Audio.get("WooHit").play();

				if(selected==BUTTONS-1)
				{
					Main.sleep(150);
					quit();
				}
				else
				{
					start = selected;
				}
			}
			else if(e.getKeyCode()==KeyEvent.VK_ESCAPE)
			{
				quit();
			}
		}
	}
	private void quit()
	{
		System.exit(0);
	}
	@Override
	public void keyReleased(java.awt.event.KeyEvent e)
	{}
	@Override
	public void keyTyped(java.awt.event.KeyEvent e)
	{}
}
