package game;

import java.awt.Font;
import java.awt.Point;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;

import javax.imageio.ImageIO;
import com.jogamp.opengl.GL2;

import audio.Audio;

import com.jogamp.opengl.util.awt.TextRenderer;

import graphics.BlendMode;
import graphics.GFX;
import mechanics.Entity;
import mechanics.Scene;
import calc.V3D;

public class Overlay extends Entity
	{


	public static String targetString;
	static String talkString;
	public static int stringPos, talkTimer;
	
	static final int width = 320;
	static final int height = 240;
	static TextRenderer courierNew12;
	static TextRenderer arialBlack16;
	static
		{
		courierNew12 =  new TextRenderer(new Font("Courier New", Font.PLAIN, 12));
		courierNew12.setColor(1,1,1,1);
		courierNew12.setSmoothing(false);
		
		arialBlack16 =  new TextRenderer(new Font("Arial Black", Font.PLAIN, 16));
		arialBlack16.setColor(1,1,1,1);
		arialBlack16.setSmoothing(false);
		}
	
	static double white;

	public Overlay(V3D pos)
		{
		super(pos);
		white = 0;
		setDepth(Integer.MIN_VALUE+9);
		}
	
	@Override
	public void render()
		{
		GL2 gl = GFX.gl;
		BlendMode.NORMAL.set();
		
		float x1 = (float)View.x, 				y1 = PataController.GND_Y-64, 
			  x2 = (float)(View.x+View.width),  y2 = PataController.GND_Y-0;
		
		gl.glBegin(GL2.GL_TRIANGLE_STRIP);
		gl.glColor4f(0,0,0,0);
		gl.glVertex2f(x1, y1);
		gl.glVertex2f(x2, y1);
		gl.glColor4f(0,0,0,0.25f);
		gl.glVertex2f(x1, y2);
		gl.glVertex2f(x2, y2);
		gl.glColor4f(1,1,1,1);
		gl.glEnd();
		
		gl.glColor4fv(GFX.BLACK,0);
		GFX.drawRectangle((float)View.x, PataController.GND_Y, (float)(View.x+View.width), (float)(View.y+View.height+6));
		
		defineOverlayMatrix();
		if(talkString!=null)
			GFX.drawRectangle(0,208,320,240);
		
		if(PataController.me.validateUserInput!=0)
			{
			float col = (float)(PataController.beatTimer%15)/15;
			if(PataController.me.userInputOdd ^ PataController.me.currentBeat%8<4)
				gl.glColor4f(0, 1-col/2, 0, 1-col);
			else
				gl.glColor4f(1-col/2, 0, 0, 1-col);
			GFX.drawTexture(GFX.tex("circle64"), 0, width-8, 8, 0, GFX.CENTER, (1-col/2), (1-col/2), 0);
			if(PataController.me.userInputOdd ^ PataController.me.currentBeat%8<4)
				gl.glColor4f(0, 1-col/2, 0, 1);
			else
				gl.glColor4f(1-col/2, 0, 0, 1);
			GFX.drawTexture(GFX.tex("outline128"), 0, width-8, 8, 0, GFX.CENTER, 0.0125f+(1-col/2)/2, 0.0125f+(1-col/2)/2, 0);
			gl.glColor4fv(GFX.WHITE,0);
			if(PataController.me.userInputOdd ^ PataController.me.currentBeat%8<4)
				GFX.drawTexture(GFX.tex(String.valueOf(1+PataController.me.currentBeat%4)), 0, width-12, 12, 0, GFX.CENTER, 1-col, 1-col, 0);
			}
		
		gl.glColor4fv(GFX.BLACK,0);
		for(SpeechBubble b:SpeechBubble.bubbles)
			{
			b.bounds = courierNew12.getBounds(b.targetString);
			Rectangle2D bounds = b.bounds;
			Point p = b.getPositionOnScreen(width, height);
			GFX.drawRectangle((float)(p.x-bounds.getWidth()/2-4), (float)(p.y-bounds.getHeight()/2-4), (float)(p.x+bounds.getWidth()/2+4), (float)(p.y+bounds.getHeight()/2+4));
			}
		
		textBegin(arialBlack16);
		String str = "X"+(int)((PataController.me.numValidMeasures+1)/2);
		if(PataController.me.validateUserInput==2)
			str+="/5";
		textDraw(arialBlack16,(int) (width-6-arialBlack16.getBounds(str).getWidth()),48,str);
		textEnd(arialBlack16);
		
		textBegin(courierNew12);
		if(talkString!=null)
			textDraw(courierNew12,8,216,talkString);
		for(SpeechBubble b:SpeechBubble.bubbles)
			{
			Rectangle2D bounds = b.bounds;
			Point p = b.getPositionOnScreen(width, height);
			textDraw(courierNew12,(int)(p.x-bounds.getX()-bounds.getWidth()/2),(int)(p.y-bounds.getHeight()/2),b.talkString);
			}
		if(Root.badCount>1)
			{
			textDraw(courierNew12, 0, 0, "BAD FRAMERATE: "+Root.frameRate,0);
			}
		
		textEnd(courierNew12);
		float w = (float)Math.abs(white);
		if(w>0.01)
			{
			gl.glColor4f(w,w,w,1);
			if(white>0)
				BlendMode.ADD.set();
			else
				BlendMode.SUBTRACT.set();
			GFX.drawRectangle(0, 0, width, height);
			BlendMode.NORMAL.set();
			}
		
		}
	
	@Override
	public void step()
		{
		if(Overlay.targetString!=null)
			{
			Pinkie.flagHolder.headImg=0;
			
			if(Intro.me != null && Intro.me.pony!=null)
				talkTimer = Intro.me.pony.talkTimer;
			else
				{
				talkTimer+=3;
				if(talkTimer>=15)
					talkTimer=0;
				if(talkTimer<7)
					Pinkie.flagHolder.headImg=1;
				}
			if(talkTimer%3==0)
				{
				if(talkTimer%6==0)
					{
					Audio.get("talk1").play();
					talkString+=targetString.charAt(stringPos);
					stringPos++;
					if(stringPos>=targetString.length())
						{
						if(Intro.me!=null && Intro.me.pony!=null)
							{Intro.me.pony.talking=false;
							Intro.me.pony.talkTimer=0;}
						talkTimer=0;
						targetString=null;
						Pinkie.flagHolder.headImg=0;
						}					
					}
				else
					Audio.get("talk2").play();
				}
			}
		}
	/**Use null to stop talking*/
	public static void talk(String string)
		{
		if(string==null)//Stop talking
			{
			Overlay.talkString = null;
			Overlay.targetString = null;
			Overlay.stringPos=0;
			}
		else
			{
			targetString = string;
			Overlay.talkString = "";
			stringPos=0;
			}
		}
	
	public static void textDraw(TextRenderer renderer, int x, int y, String string)
		{
		textDraw(renderer,x,y,string,1);
		}
	public static void textDraw(TextRenderer renderer, int x, int y, String string, float colorMultiplier)
		{
		y+=renderer.getFont().getSize();
		renderer.setColor(1f*colorMultiplier, 0.6f*colorMultiplier, 0.85f*colorMultiplier, 1f);
		renderer.draw3D(string, x+1, -y, 0, 1);
		renderer.draw3D(string, x-1, -y, 0, 1);
		
		renderer.setColor(colorMultiplier, colorMultiplier, colorMultiplier, 1);
		renderer.draw3D(string, x, -y, 0, 1);
		}

	public static void textBegin(TextRenderer renderer)
		{
		GFX.gl.glPushMatrix();
		GFX.gl.glScaled(1, -1, 1);
		//GFX.gl.glTranslated(0,height,0);
		
		GFX.gl.glMatrixMode(GL2.GL_PROJECTION);
		GFX.gl.glPushMatrix();
		GFX.gl.glMatrixMode(GL2.GL_MODELVIEW);
		renderer.begin3DRendering();
		GFX.gl.glMatrixMode(GL2.GL_MODELVIEW);
		}

	public static void textEnd(TextRenderer renderer)
		{
		renderer.end3DRendering();
		//Simply popping a matrix isn't enough. It must be redefined.
		//defineOverlayMatrix();
		GFX.gl.glMatrixMode(GL2.GL_PROJECTION);
		GFX.gl.glPopMatrix();
		GFX.gl.glMatrixMode(GL2.GL_MODELVIEW);
		GFX.gl.glPopMatrix();
		}

	public static void defineOverlayMatrix()
		{
		GFX.gl.glMatrixMode(GL2.GL_MODELVIEW);
        GFX.gl.glLoadIdentity(); 
		GFX.gl.glOrthof(0, width, height, 0, -6400, 6400);
		}
	
	void screenshot()
		{
		GL2 gl = GFX.gl;

		gl.glReadBuffer(GL2.GL_BACK);
		ByteBuffer pixels = ByteBuffer.allocateDirect(Scene.width*Scene.height*4);
		gl.glReadPixels(0, 0, Scene.width, Scene.height, GL2.GL_RGBA, GL2.GL_UNSIGNED_BYTE, pixels);
		BufferedImage bi = new BufferedImage(Scene.width, Scene.height, BufferedImage.TYPE_4BYTE_ABGR);
		
		pixels.rewind();
		byte[] data = new byte[Scene.width*Scene.height*4];
		for(int j = Scene.height-1; j>=0; j--)
			{
			for(int i = 0; i<Scene.width; i++)
				{
				data[(j*Scene.width+i)*4  ] = pixels.get();
				data[(j*Scene.width+i)*4+1] = pixels.get();
				data[(j*Scene.width+i)*4+2] = pixels.get();
				data[(j*Scene.width+i)*4+3] = pixels.get();
				}
			}
		
		bi.getRaster().setDataElements(0, 0, Scene.width, Scene.height, data);
		
		try {ImageIO.write(bi, "PNG", new File("out.png"));}
		catch (IOException e)
			{e.printStackTrace();}
		}
	}
