package game;

import graphics.GFX;
import audio.Audio;
import audio.StupidClip;

import calc.Calc;
import calc.V3D;
import mechanics.Entity;
import mechanics.Scene;

public class ParaspriteSphere extends Entity implements Enemy
	{
	static final int MAX_PARASPRITES = 48;
	static ParaspriteSphere me;
	Parasprite[] parasprites;
	int numParasprites;
	int phase;
	boolean ground;
	double radius;
	double dir;
	int life;
	
	boolean rolledAway = false;
	int goalX;
	
	boolean dangerous;
	/**calculate parasprite initial cloud target locations*/
	double d=0,s=0;
	int timeout;
	
	double subPhase;
	int fallBeat;
	StupidClip sound;
	
	public ParaspriteSphere(V3D pos)
		{
		super(pos);
		visible = false;
		life = 3;
		sound = Audio.get("parasprite");
		sound.setGain(0);
		sound.setLooping(true);
		sound.play();
		PataController.addEnemy(this);
		me=this;
		dangerous = false;
		timeout=0;
		ground = true;
		parasprites = new Parasprite[MAX_PARASPRITES];
		numParasprites = 0;
		phase = 0;
		radius = 0;
		dir = 0;
		}
	
	@Override
	public void destroy()
		{
		super.destroy();
		sound.stop();
		me = null;
		}
	
	@Override
	public void step()
		{
		if(timeout>0)
			timeout-=1;
		
		if(phase==0)
			{
			if(Scene.fourstage==0)
				{
				V3D p;
				p = new V3D(pos.x()+(Math.random()-0.5)*320,PataController.GND_Y-480,0);
				//p = pos.clone();
				parasprites[numParasprites] = new Parasprite(p,d+(Math.random()/5),s+(Math.random()/10-0.05));
				if (s==0)
					{s=0.2;}
				else 
					{
					int segments;
					if(Math.abs(s-0.2)<0.01)
						segments = 3;
					else if(Math.abs(s-0.4)<0.01)
						segments = 6;
					else if(Math.abs(s-0.6)<0.01)
						segments = 9;
					else if(Math.abs(s-0.8)<0.01)
						segments = 13;
					else
						segments = 16;
					d+=Math.PI*2/segments;
					if(d>=Math.PI*2-0.01)
						{d=0;
						s+=0.2;}
					}
				numParasprites++;
				radius+=1;
				if(numParasprites>=MAX_PARASPRITES)
					{
					phase = 1;
					timeout=120;
					}
				}
			}
		if(phase == 0 || phase==1)//Roll to front of pinkies
			{
			if(pos.x()<Pinkie.flagHolder.pos.x()+216)
				{
				pos.p[0]+=4;
				if(radius>0)
					dir-=(4d/radius);
				}
			if(pos.x()>Pinkie.flagHolder.pos.x()+224)
				{
				pos.p[0]-=4;
				if(radius>0)
					dir+=(4d/radius);
				}
			if(phase==1 && timeout<=0 && ((PataController.me.currentBeat%4)+((PataController.me.userInputOdd ^ PataController.me.currentBeat%8<4)?0:4))==3)
				{
				if(Math.random()<0.5)//JUMP
					{
					subPhase=0;
					phase=2;
					alarm[0] = 30;
					fallBeat = PataController.me.currentBeat+10;
					}
				else//ROLL
					{
					subPhase=0;
					phase=6;
					speed.set(0,-3,0);
					ground = false;
					fallBeat = PataController.me.currentBeat+10;
					}
				}
			}
		else if(phase==2)//Prepare for jump
			{
			subPhase+=1d/30;
			xScale = (float)(1+Math.sin(Math.PI*subPhase)/2);
			yScale = (float)(1-Math.sin(Math.PI*subPhase)/2);
			}
		else if (phase==3)//Float over pinkies
			{
			dir+=Calc.toRad(3);
			xScale=(float)(1+speed.y()/24);
			yScale=(float)(1-speed.y()/16);
			if(speed.y()<0)
				speed.p[1]+=0.5;
			if(pos.x()>Pinkie.flagHolder.pos.x())
				{
				pos.p[0]-=4;
				speed.p[0]-=0.25;
				}
			else
				{
				pos.p[0]+=4;
				speed.p[0]+=0.25;
				}
			if(PataController.me.currentBeat>=fallBeat)
				{
				phase=4;
				dangerous = true;
				}
			}
		else if(phase==4)//Fall onto pinkies
			{
			xScale=0.5f;
			yScale=0.5f;
			speed.p[1]+=0.45;
			if(pos.y()>PataController.GND_Y+92)
				{
				alarm[1]=15;
				xScale=0.5f;
				yScale=2f;
				speed.set(4,-6,0);
				phase=5;
				for(int i=0; i<numParasprites; i++)
					{
					parasprites[i].speed.add((Math.random()-0.5)*32,-Math.random()*16,0);
					}
				pos.p[1] = PataController.GND_Y-32;
				pos.p[0] += 32;
				}
			}
		else if(phase==5)//Roll back into place
			{
			dir-=Calc.toRad(6);
			speed.p[1]+=0.18;
			if(xScale>1)
				xScale-=0.05;
			if(yScale<1)
				yScale+=0.05;
			if(xScale<1)
				xScale+=0.05;
			if(yScale>1)
				yScale-=0.05;
			if(pos.y()>PataController.GND_Y-radius*yScale && speed.y()>0)
				{
				xScale=1;
				yScale=1;
				phase=1;
				timeout=120;
				ground=true;
				speed.set(0,0,0);
				}
			}
		else if(phase==6)//Bounce out
			{
			speed.p[1]+=0.2;
			if(pos.y()>PataController.GND_Y && speed.y()>0)
				{
				speed.p[1]*=-1;
				}
			if(pos.x()<Pinkie.flagHolder.pos.x()+316)
				pos.p[0]+=4;
			else
				pos.p[0]+=2;
			dir+=(4d/radius);
			if(PataController.me.currentBeat>=fallBeat)
				{
				speed.set(-16,0,0);
				phase=7;
				pos.p[1]=PataController.GND_Y-32;
				dangerous=true;
				ground = false;
				xScale=2;
				yScale=2;
				}
			}
		else if (phase==7)//Roll smash
			{
			dir+=(10d/radius);
			if(pos.x()<View.x-640)
				{
				phase=1;
				timeout=150;
				dangerous=false;
				ground=true;
				xScale=1;
				yScale=1;
				speed.set(0,0,0);
				pos.p[0]=View.x+View.width+128;
				pos.p[1]=PataController.GND_Y-32;
				for(int i=0; i<numParasprites; i++)
					{
					parasprites[i].pos.set(pos);
					}
				}
			}
		else if(phase == 8)//Roll away
			{
			if(pos.x()<goalX)
				{
				pos.p[0]+=6;
				if(radius>0)
					dir-=(6d/radius);
				}
			if(Pinkie.flagHolder.pos.x()>goalX-384)
				phase=0;
			}
		
		
		if(ground)
			{
			pos.y(PataController.GND_Y-radius*yScale);
			}
		
		sound.setGain((double)numParasprites/MAX_PARASPRITES);
		if(numParasprites>0)
			{
			int accumX=0;
			int accumY=0;
			
			double accumH=0;
			double accumV=0;
			for(int i=0; i<numParasprites; i++)
				{
				Parasprite p = parasprites[i];
				accumX+=p.pos.x();
				accumY+=p.pos.y();
				accumH+=p.speed.x();
				accumV+=p.speed.y();
				p.targetX = pos.x()+xScale*p.sp*radius*Math.cos(dir+p.dir);
				p.targetY = pos.y()-yScale*p.sp*radius*Math.sin(dir+p.dir);
				}
			accumX/=numParasprites;
			accumY/=numParasprites;

			accumH/=numParasprites;
			accumV/=numParasprites;
			sound.setPan(Calc.limit(accumH/8,-1,1));
			sound.setFrequency(Calc.limit(1+accumV/5, 0.2, 1.6));
			for(Pinkie p: Pinkie.pinkies)
				{
				if(Calc.dist(accumX,accumY,p.pos.x(),p.pos.y())<Pinkie.RADIUS+radius/2)
					{
					int hit = 0;
					int panim = p.anim;
					if(panim!=Pinkie.JUMP && phase!=4 && phase!=0 && p!=Pinkie.flagHolder)
						{
						if(dangerous && panim!=Pinkie.DUCK)
							{
							hit++;
							if(phase==7)
								p.hit(1);
							else
								p.hit(2);
							}
						else if(panim==Pinkie.RUN)
							{
							hit++;
							p.hit(0);
							hit(PataController.me.isFever()?2:1);
							}
						else if(panim==Pinkie.DUCK && phase==7)
							{
							for(int i=0; i<numParasprites; i++)
								{
								parasprites[i].speed.set(16+2*Math.random(),-3-5*Math.random(),0);
								}
							speed.set(4,-4,0);
							pos.x(p.pos.x()+128);
							dangerous = false;
							phase=5;
							Audio.get("blow1").play();
							}
						
						if(hit!=0)
							Audio.get("blow2").play();
						}
					}
				}
			}
		super.step();
		}
	private void hit(int damage)
		{
		if(phase == 8 && pos.x()>goalX-32)
			phase=0;
		for(int i=0; i<numParasprites; i++)
			{
			Parasprite p = parasprites[i];
			p.speed.add(damage*8*(Math.random()-0.5),damage*8*(Math.random()-0.5),0);
			}
		life -= damage;
		if(life<=0 && numParasprites>0)
			{
			parasprites[numParasprites-1].destroy();
			new Explosion(parasprites[numParasprites-1].pos.clone());
			parasprites[numParasprites-1]=null;
			numParasprites--;
			radius-=0.66;
			life=2;
			}
		if(numParasprites<24 && !rolledAway)
			{
			rolledAway = true;
			phase = 8;
			goalX = (int)pos.x()+960;
			xScale=1;
			yScale=1;
			ground=true;
			speed.set(0,0,0);
			Rock.rockPile((int)pos.x()+480);
			Scenery.spawn((int)pos.x()+640, "hill");
			}
		if(numParasprites==0)
			destroy();
		
		}

	@Override
	public void alarm(int index)
		{
		if (index==0)
			{
			ground=false;
			speed.set(0,-16,0);
			phase=3;
			}
		else if(index==1)
			dangerous = false;
		}

	@Override
	public V3D getPosition()
		{
		return pos;
		}
	@Override
	public void render()
		{
		GFX.unitSphere2.render(pos, 16);
		}
	}
