package game;

import io.IOH;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Arrays;

import audio.Audio;
import audio.JavaSoundListener;
import audio.StupidClip;

import calc.V3D;
import main.Main;
import mechanics.DestructionListener;
import mechanics.Entity;
import mechanics.Scene;

public class PataController extends Entity implements KeyListener, JavaSoundListener, DestructionListener
	{
	public static PataController me;
	/**The list of enemies is used only to identify whether an enemy is nearby(this changes the View'sss size).*/
	private static ArrayList<Enemy> enemyList = new ArrayList<Enemy>();
	
	static String[] drums = {"GoHit",null,"HeyHit"};
	static String[] pinkie1 = {"Go1","Whee1","Hey1"};
	static String[] pinkie2 = {"Go2","Whee2","Hey2"};

	static final int GND_Y = 208;
	
	private static String musicName = null;
	int sweepRetrig = 8;

	
	int averageDelayTime = Audio.getLine().getAverageDelay();
	/**Updated on each beat, 
	 * this is an estimate time in milliseconds time for the next beat sound play call.
	 * It should be compared against nanotime.*/
	long nextBeatTime;
	/**Updated on each beat, 
	 * this is the time in milliseconds time for the previous beat sound play call.
	 * It should be compared against nanotime/1000000.*/
	long previousBeatTime;
	/**Updated on each 4th beat(number 3),
	 * this is the time in milliseconds for the current measure's user input end.
	 * Should be compared against nanotime/1000000.
	 * It is -1 during the period from the end of the input until the next assignment.*/
	long userInputEndTime;
	
	static boolean marchEnabled = true;
	static boolean attackEnabled = true;
	static boolean defendEnabled = true;
	
	/**Current music pattern index*/
	int currentPattern;
	/**Currently playing clip*/
	StupidClip currentClip;
	
	boolean ignoreStopEvent;
	
	/**Increases with beat(), starting at 0*/
	int currentBeat;
	
	/**True  if the user starts input on a totalNumBeats%8 == 4,
	 * false if the user starts input on a totalNumBeats%8 == 0*/
	boolean userInputOdd;
	
	/**Whether we already won the level*/
	static boolean win;
	/**Whether to listen to user input at all*/
	boolean control;
	/**Whether to validate and respond to user input: 0 auto-fail 1 normal gameplay, 2 intro testing*/
	int validateUserInput;
	
	/**Increases approx. beatTime/2 milliseconds before a new user input measure starts, iff the user input is still valid*/
	int numValidMeasures;
	/**True as long as the user has not messed up.*/
	boolean userInputValid;
	/**The current user input. 4 values, 0-2.*/
	int[] userInput;
	/**The current user input errors.*/
	int[] userDiffs;
	
	/**The time for a single drum beat in ms*/
	int beatTime = 500;
	/**Maximum allowed timing error for user drum beats in ms*/
	int lenience = 240;
	
	/**Set to true when we were going and then failed*/
	boolean failing;
	/**Is set to the index on a beat() call. The purpose is to schedule certain events that must happen on this thread.*/
	private int beatOccured;
	
	/**Set to true when the fail sound plays. The purpose is to schedule a fail on this thread(step event).*/
	private boolean failOccured;
	private int stampOccured;
	/**The overall volume. Decreases when dead.*/
	private double musicVolume;
	/**The predefined gain for the current music pattern.*/
	private double currentGain;
	/**Whether all Pinkies except flagHolder have died*/
	private boolean dead;
	/**Whether we won and the fanfare stopped.*/
	private boolean won;
	/**Goes from 0 to 30 approx. Is synchronized with beats.*/
	public static int beatTimer;
	
	static boolean skipIntro = false;
	static int levelIndex = 2;
	/**X the flag bearer must pass to win view will never see more than 160px beyond this(in world coordinates)*/
	public static int goalX = 0;
	
	public PataController(V3D pos)
		{
		super(pos);
		won = false;
		drums[1] = "WheeHit";//Changed in credits
		musicVolume = 1;
		stampOccured = -1;
		me = this;
		dead = false;
		
		marchEnabled = true;
		attackEnabled = true;
		defendEnabled = true;

		Main.canvas.addKeyListener(this);
		Audio.addJavaSoundListener(this);
		failOccured = false;
		control = true;
		validateUserInput = 1;
		beatTimer = 0;
		
		new View(new V3D(0,0,0));
		new Overlay(new V3D(0,0,0));
		
		startLevel(levelIndex);
		
		beatOccured = -1;
		ignoreStopEvent = false;
		failing = false;
		userInput = new int[4];
		userDiffs = new int[4];
		Arrays.fill(userInput, -1);
		userInputValid = true;
		userInputEndTime=-1;
		numValidMeasures = 0;
		currentBeat = -1;
		currentPattern = 0;
		currentClip = Audio.get(getMusicName(currentPattern));
		currentClip.setPositionListener(this, (beatTime*currentClip.getFrameRate())/1000);
		currentClip.play();
		beat();
		}

	private void startLevel(int index)
		{
		win = false;
		levelIndex = index;
		new Pinkie(new V3D(160,GND_Y-Pinkie.RADIUS,0),true);
		for(int i=0; i<8; i++)
			new Pinkie(new V3D(48+(i+Math.random())*24,GND_Y-Pinkie.RADIUS,0),false);
		if(index==0)
			new LevelController0(V3D.ZERO.clone());
		else if(index==1)
			new LevelController1(V3D.ZERO.clone());
		else if(index==2)
			new LevelController2(V3D.ZERO.clone());
		else if(index==3)
			new LevelController3(V3D.ZERO.clone());
		}

	private String getMusicName(int index)
		{
		return musicName+"-"+index;
		}
	
	@Override
	public void step()
		{
		super.step();
		if(isDestroyed())
			return;
		long currentTime = System.nanoTime()/1000000;
		beatTimer++;
		
		if(!dead)
			{
			dead = true;
			for(Pinkie p: Pinkie.pinkies)
				{
				if(p!=Pinkie.flagHolder && !p.isDestroyed())
					dead = false;
				}
			if(LevelController3.me!=null)
				dead = false;
			if(dead)
				{
				fail(6);
				control = false;
				alarm[0] = 120;
				}
			}
		else
			{
			if(musicVolume>0)
				{
				musicVolume-=1d/60;
				if(musicVolume<=0)
					{
					musicVolume = 0;
					Pinkie.flagHolder.mode = Pinkie.ATTACK;
					Pinkie.flagHolder.headImg = 2;
					Pinkie.flagHolder.eyeImg = 0;
					Pinkie.flagHolder.targetX = Integer.MIN_VALUE;
					}
				currentClip.setGain(musicVolume*currentGain);
				currentClip.setFrequency(musicVolume);
				}
			else
				{
				Pinkie.flagHolder.mode = Pinkie.ATTACK;
				Pinkie.flagHolder.targetX = Integer.MIN_VALUE;
				musicVolume=0;
				if(Overlay.white>-1)
					Overlay.white-=1d/60;
				else
					Overlay.white=-1;
				}
			}
		
		if(Pinkie.flagHolder.pos.x()>goalX && LevelController3.me==null)
			{
			if(!win)
				{
				this.control = false;
				this.validateUserInput = 0;
				Pinkie.allWin();
				win = true;
				Audio.get("Cheer").play();

				this.ignoreStopEvent = true;
				this.currentClip.stop();
				this.ignoreStopEvent = false;
				
				this.currentClip = Audio.get("Fanfare"); 
				currentClip.play();
				}
			}
		
		if(userInputEndTime!=-1 && currentTime>=userInputEndTime)//User input for this measure ended
			{
			userInputEndTime=-1;
			
			int beatIndex = (currentBeat-3)%8;
			if(userInputOdd){beatIndex-=4;}
			if (beatIndex<0) beatIndex+=8;
			if(beatIndex>=8) beatIndex-=8;
			
			
			if(beatIndex==0)
				{
				//Check for holes
				int i=0;
				while(i<4 && userInputValid)
					{
					if(userInput[i]==-1 || validateUserInput==0)
						fail(i==0?4:0);//Missed a beat(0) or missed everything(4)
					i++;
					}
				if(userInputValid && validateUserInput!=3 &&  
				   !(userInput[0]==0 && userInput[1]==0 && userInput[2]==1 && userInput[3]==0 && attackEnabled) &&
				   !(userInput[0]==1 && userInput[1]==1 && userInput[2]==1 && userInput[3]==0 && marchEnabled) &&
				   !(userInput[0]==2 && userInput[1]==2 && userInput[2]==1 && userInput[3]==0 && defendEnabled))
					fail(5);
				
				if(validateUserInput==2 && userInputValid)
					{
					Intro.successfulInput();
					}
				}
			
			if(userInputValid)
				{
				numValidMeasures++;
				if(beatIndex==4)//About to start new input
					{					
					Arrays.fill(userInput, -1);
					}
				else if(validateUserInput==1)
					{
					Pinkie.allInputPattern(userInput);
					}
				else if(validateUserInput==3)
					{
					Pinkie.allParty();
					}
				}
			else//Fail
				{
				userInputOdd = ((currentBeat+1)%8 == 4);
				userInputValid = true;
				Arrays.fill(userInput, -1);
				}
			}
		
		if(failOccured)
			{
			Pinkie.allFail();
			failOccured = false;
			}
		if(stampOccured != -1)
			{
			Stamp s = new Stamp(new V3D(32+(stampOccured%4)*(320-160)/3,16,0),String.valueOf(1+(stampOccured%4)));
			s.followScreen = true;
			s.alpha = 0.5f;
			s = new Stamp(new V3D(32+(stampOccured%4)*(320-160)/3,48,0),stampOccured==3?"pon":"pata");
			s.alpha = 0.5f;
			s.followScreen = true;
			stampOccured=-1;
			}
		if(beatOccured!=-1)
			{
			if(beatOccured%2==0)
				beatTimer =0;
			Pinkie.allBeat(beatOccured,numValidMeasures);
			Intro.beat(beatOccured,numValidMeasures);
			beatOccured = -1;
			}
		
		if(won)
			endLevel(END_WIN);
		else if(IOH.checkKeyPressed(KeyEvent.VK_ESCAPE))
			endLevel(END_QUIT);
		}
	
	@Override
	public void alarm(int i)
		{
		if(i==0)
			{
			endLevel(END_DEAD);
			}
		}

	private void fail(int type)
		{
		//
		if(numValidMeasures>0)
			failing = true;
		numValidMeasures = 0;
		userInputValid = false;
		Intro.failInput(type);
		System.out.println("fail "+type);
		}

	@Override
	public void soundStop(StupidClip c, int milliseconds)
		{
		if(win && c == currentClip && !ignoreStopEvent)
			{
			won = true;
			}
		else if (c.getName().endsWith("-"+currentPattern) && !ignoreStopEvent)
			{
			if(musicName.equals("Tar") && currentPattern!=0)
				{
				System.out.println("Teh edn");
				LevelController3.portal();
				}
			else
				{
				String currentMusic = c.getName();
				
				if(numValidMeasures>9)
					{
					if((numValidMeasures-10)%sweepRetrig==0)
						{
						String fever = "Fever"+(int)(1+Math.random()*4);
						Audio.get(fever).setGain(0.66);
						Audio.get(fever).play();
						}
					
					if(musicName.equals("Freidus"))
						{
						if((numValidMeasures-10)%48 == 8 || (numValidMeasures-10)%24 == 12)
							currentPattern = 5;
						else if((numValidMeasures-10)%24 == 16)
							currentPattern = 6;
						else if((numValidMeasures-10)%24 == 20)
							currentPattern = 7;
						else
							currentPattern = 4;
						}
					if(musicName.equals("McFest"))
						{
						if((numValidMeasures-10)%24 == 16 || (numValidMeasures-10)%24 == 20)
							currentPattern = 5;
						else if((numValidMeasures-10)%24 == 8)
							currentPattern = 6;
						else if((numValidMeasures-10)%24 == 12)
							currentPattern = 7;
						else
							currentPattern = 4;
						}
					else
						{
						currentPattern = 4+((numValidMeasures-10)/4)%4;
						}
					}
				else if(numValidMeasures>1)
					{
					if(musicName.equals("Tar"))
						currentPattern = 1;
					else if(currentMusic.endsWith("-2"))
						currentPattern = 3;
					else if(!currentMusic.endsWith("-3"))
						currentPattern = 2;
					}
				else
					{
					if(currentMusic.endsWith("-0") && !musicName.equals("Tar"))
						currentPattern = 1;
					else
						currentPattern = 0;
					}
				
				double g = 1;
				currentClip = Audio.get(getMusicName(currentPattern));
				currentClip.play();
				if(musicName.equals("Parasprite"))
					{
					if(numValidMeasures>=10)
						g = 0.92;
					else
						g = 0.6;
					}
				if(musicName.equals("McFest"))
					{
					if(numValidMeasures>=10)
						g = 0.92;
					else
						g = 0.825;
					}
				currentGain = g;
				g *= musicVolume;
				currentClip.setPositionListener(this, (beatTime*currentClip.getFrameRate())/1000);
				
				currentClip.setGain(g);
				currentClip.setFrequency(musicVolume);
				
				beat();
				}
			}
		}


	static final int END_WIN   = 0;
	static final int END_DEAD  = 1;
	static final int END_QUIT  = 2;
	static final int END_FINAL = 3;
	public void endLevel(int type)
		{
		if(type==END_FINAL || type==END_WIN)
			Menu.setProgress(levelIndex+2) ;
		
		Overlay.talk(null);
		Pinkie.pinkies = new ArrayList<Pinkie>();
		Pinkie.flagHolder = null;
		ignoreStopEvent = true;
		currentClip.stop();
		ignoreStopEvent = false;
		Overlay.white = 0;
		Entity[] entities = Scene.getInstances();
		for(Entity e: entities)
			{
			if(!e.isDestroyed())
				e.destroy();
			}
		new Menu(new V3D());
		}

	@Override
	public void keyPressed(KeyEvent e)
		{
		int timeSincePrev = (int) (System.nanoTime()/1000000-previousBeatTime);
		int timeUntilNext = (int) (nextBeatTime-System.nanoTime()/1000000);
		
		//Index of the nearest beat and time diff:
		int beatIndex = currentBeat%8;
		if(userInputOdd){beatIndex-=4;}
		int diff;
		if(timeSincePrev<timeUntilNext)
			{diff = timeSincePrev;}
		else
			{diff = timeUntilNext;
			beatIndex++;}
		if (beatIndex<0) beatIndex+=8;
		if(beatIndex>=8) beatIndex-=8;

		
		if(control && (beatIndex<4))
			{
			int key = e.getKeyCode();
			int beat=-1;
			String stamp = "";
			if(key==KeyEvent.VK_A)
				{beat = 0;
				stamp = "pon";
				Intro.aPressed=true;}
			else if(key==KeyEvent.VK_S)
				{beat = 1;
				stamp = "pata";
				Intro.sPressed=true;}
			else if(key==KeyEvent.VK_D)
				{beat = 2;
				stamp = "chaka";
				Intro.dPressed=true;}
			if(beat!=-1)
				{
				if(validateUserInput!=3)
					new Stamp(new V3D(View.x+48+beat*(View.width-96)/2,View.y+(beat==1?48:80),0),stamp);
				Audio.get(drums[beat]).play();
								
				if(userInputValid)
					{
					if(beatIndex>=4)//Invalid index(One and only way to fail while not your turn)
						fail(2);
					else if(diff>lenience)//Too inaccurate
						fail(1); 
					else if(userInput[beatIndex]!=-1)//Already input beat here
						fail(3);
					else
						{
						int i=0;
						while(i<beatIndex && userInputValid)
							{
							if(userInput[i]==-1)//Missed a beat
								{
								fail(i==0?6:0);//Started on wrong(6) or missed one(0)
								}
							i++;
							}
						if(userInputValid) //Still valid!
							{
							userInput[beatIndex] = beat;
							userDiffs[beatIndex] = diff;
							}
						}
					}
				}
			}
		}

	@Override
	public void keyReleased(KeyEvent e)
		{}

	@Override
	public void keyTyped(KeyEvent e)
		{}

	@Override
	public void soundAction(StupidClip c, int position)
		{
		int nextPosition = position+(beatTime*c.getFrameRate())/1000;
		if(nextPosition<c.getFrameLength()-1000)
			c.setPositionListener(this, nextPosition);
		beat();
		}
	/**Will in many cases be called from the AudioLine's thread.*/
	private void beat()
		{
		
		previousBeatTime = System.nanoTime()/1000000;
		nextBeatTime = previousBeatTime + beatTime;
		currentBeat++;//It is now on the new index
		
		int beatIndex = currentBeat%8;
		if(userInputOdd){beatIndex-=4;}
		if (beatIndex<0) beatIndex+=8;
		if(beatIndex>=8) beatIndex-=8;

		int eightIndex = beatIndex;
		
		if(beatIndex>=4 && numValidMeasures>0)
			{
			beatIndex-=4;
			if(userInput[beatIndex]!=-1)
				{
				if(validateUserInput!=3)
					{
					String[] src;
					if(beatIndex==3 || beatIndex==1)
						src = pinkie2;
					else
						src = pinkie1;
					StupidClip c = Audio.get(src[userInput[beatIndex]]);
					c.setGain(0.75);
					c.play();
					}
				Audio.get(drums[userInput[beatIndex]]).play();
				}
			}
		
		if((this.validateUserInput==2 || numValidMeasures==0) && eightIndex<4 && validateUserInput!=0 &&
				(this.currentPattern==0 || validateUserInput!=3))
			{
			if(beatIndex==0)
				{
				Audio.get("metronome1").play();
				Audio.get("metronome1").setGain(musicVolume);
				}
			else
				{
				Audio.get("metronome2").play();
				Audio.get("metronome2").setGain(musicVolume);
				}
			if(validateUserInput==2)
				{
				stampOccured = beatIndex;
				}
			}
		
		beatOccured = beatIndex;
		
		if(currentBeat%4 == 0 && validateUserInput!=3 && validateUserInput!=0)
			{
			if(numValidMeasures==1 || numValidMeasures==9 || (numValidMeasures-10)%sweepRetrig==sweepRetrig-1)
				Audio.get(musicName+"-SweepUp").play();
			if(numValidMeasures==2 || numValidMeasures==10 || (numValidMeasures-10)%sweepRetrig==0)
				Audio.get(musicName+"-SweepDown").play();
			}
			
		
		if(currentBeat%4 == 3)
			{
			userInputEndTime = nextBeatTime-beatTime/2;
			}
		
		if(currentBeat%4 == 0 && failing)
			{
			if(this.validateUserInput==1)
				{
				String fail = "Fail"+(int)(1+Math.random()*4);
				Audio.get(fail).play();
				failOccured = true;
				}
			if(this.validateUserInput!=0 && this.validateUserInput!=3)
				Audio.get("Fail").play();
			
			failing = false;
			
			if(validateUserInput!=3)
				{
				currentPattern = 0;
				ignoreStopEvent = true;
				currentClip.setPositionListener(null, 0);
				ignoreStopEvent = false;
				currentClip.stop();
				currentClip = Audio.get(getMusicName(currentPattern));
				currentClip.setPositionListener(this, (beatTime*currentClip.getFrameRate())/1000);
				currentClip.play();
				}
			}
		}

	public static void addEnemy(Enemy e)
		{
		enemyList.add(e);
		e.addDestructionListener(me);
		}
	public static void removeEnemy(Enemy e)
		{
		enemyList.remove(e);
		e.removeDestructionListener(me);
		}
	
	public static ArrayList<Enemy> getEnemyList()
		{
		return enemyList;
		}
	@Override
	public void destroy()
		{
		super.destroy();
		Main.canvas.removeKeyListener(this);
		Audio.removeJavaSoundListener(this);		
		}
	
	@Override
	public void instanceDestroyed(Entity e)
		{
		if(e instanceof Enemy)
			{
			enemyList.remove((Enemy)e);
			}
		}

	public boolean isFever()
		{
		return numValidMeasures>=10;
		}
	/**Set music name and load files. Delete old music, if loaded. Runs gc.*/
	public static void setMusicName(String string)
		{
		if(musicName!=null)
			Audio.unloadDir("res/music/"+musicName);
		musicName = string;
		Audio.loadDir("res/music/"+musicName);
		Runtime.getRuntime().gc();
		}
	}
