package game;


import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;

import audio.Audio;

import graphics.Armature;
import graphics.Bone;
import graphics.GFX;
import graphics.MTL;
import graphics.Model;
import calc.Calc;
import calc.V3D;

public class PonyAnimator extends SphereEntity
	{
	static final int[] FRAMES = new int[]{0,81,151,181,241,301,361,440,470,500,530,547,578,668,758,
											760,790,820,850,880,910,920,950,970}; 
	static final int CLIMBUP   = 0,
					 LOOKBACK  = 1,
					 IDLE 	   = 2,
					 POINTLEFT = 3,
					 POINTRIGHT= 4,
					 POINTUP   = 5,
					 CLIMBDOWN = 6,
					 STANDTROT = 7,
					 TROT	   = 8,
					 RUN	   = 9,
					 RUNTROT   = 10,
					 TURN	   = 11,
					 JUMP	   = 12,
					 COUNT     = 13,
					 COUNTEND  = 14,
					 DANCEBEGIN= 15,
					 DANCEJUMP = 16,
					 UPSTART   = 17,
					 UPLOOP    = 18,
					 UPEND     = 19,
					 BUTTSTART = 20,
					 BUTTLOOP  = 21,
					 BUTTEND   = 22;
	
	static
		{
		Armature armature = Armature.get("pinkieClimb.bvh");
		Bone master=armature.getBone("IK.Master");
		for(int i=0; i<armature.frames; i++)//Translate the animation down to the center.
			{
			master.setPositionData(i,
					master.getX(i)-master.offsetFromParent[0],
					master.getY(i)-master.offsetFromParent[1],
					master.getZ(i)-master.offsetFromParent[2]);
			float dy = master.getY(i);
			if(dy!=0)//Workaround because I was stupid and translated Master when making turnKick
				{
				master.setPositionData(i, master.getX(i), 0, master.getZ(i));
				for(int j=0; j<armature.bones.length; j++)
					{
					Bone b = armature.bones[j];
					if(b.parent==master)
						b.setPositionData(i, b.getX(i), b.getY(i)+dy, b.getZ(i));
					}
				}
			}
		}	
	Armature armature,hairArmature,tailArmature;
	Model model, blinkModel, hairModel, tailModel,pupilModel;
	/**References to bones, to avoid looking up in the hashmap all the time*/
	protected Bone hip,crossBack,back,neck,master,shoulderL,shoulderR,footL,footR,handL,handR,head,headExt;
	/**References to bones, to avoid looking up in the hashmap all the time*/
	private Bone hairMaster,frontHair0,frontHair1,jaw;
	/**Used only to calculate wobbling of front hair.*/
	private float lastHairZ,hairDir,hairDirSp;
	/**Body animation frame*/
	float frame,lastFrame;
	/**current animation*/
	int animationIndex;
	
	V3D c1 = new V3D();
	V3D c2 = new V3D();
	V3D c3 = new V3D();
	V3D c4 = new V3D();
	/**Arrays holding strings of TailLinks*/
	private TailLink[] tail,hair;
	
	/**Pupil rotation angles*/
	protected float eyeAngleXL=0,eyeAngleXR=0,eyeAngleZL=0,eyeAngleZR=0;
	
	private int blinkTimer = 0;
	private V3D lookAt;
	
	boolean talking;
	boolean animate;
	int talkTimer;
	private float jawX,jawY,jawZ;
	private AnimationListener listener;
	float interp;
	boolean flipVision;
	public boolean mirror;

	public PonyAnimator(V3D pos, boolean minDepth)
		{
		super(pos);
		interp = 1;
		mirror = false;
		flipVision = false;
		if(minDepth)
			setDepth(Integer.MIN_VALUE+7);
		scale = 80;
		animate = true;
		talking = false;
		gravity = new V3D(0,0,0);
		z = V3D.Y.clone().multiply(-1);
		pos.add(0,scale*(3.4-1.48),0);
		armature = Armature.get("pinkieClimb.bvh");
		hairArmature = Armature.get("pinkieHair.bvh");
		tailArmature = Armature.get("pinkieTail.bvh");
		model = Model.get("pinkie.obj",armature);
		blinkModel= Model.get("pinkieBlink.obj",armature);
		hairModel = Model.get("pinkieHair.obj",hairArmature);
		tailModel = Model.get("pinkieTail.obj",tailArmature);
		pupilModel= Model.get("pinkieEye.obj",null);
		
		//Handle individual bone settings
		
		
		master=armature.getBone("IK.Master");
		master.usePostMatrix=true;
		master.postMatrix = new float[][]{
					{1,0,0},
					{0,0,-1},
					{0,-1,0}};
		
		hip=armature.getBone("Hip");
		crossBack=armature.getBone("CrossBack");
		back=armature.getBone("Back");
		neck=armature.getBone("Neck");
		footL=armature.getBone("Foot.L");
		footR=armature.getBone("Foot.R");
		handL=armature.getBone("Hand.L");
		handR=armature.getBone("Hand.R");
		shoulderL=armature.getBone("Shoulder.L");
		shoulderR=armature.getBone("Shoulder.R");
		head=armature.getBone("Head");
		headExt=armature.getBone("HeadExt");
		
		jaw = armature.getBone("Jaw");
		jawX = jaw.getX(0);
		jawY = jaw.getY(0);
		jawZ = jaw.getZ(0);
		
		hairMaster = hairArmature.getBone("HairMaster");
		hairMaster.usePostMatrix=true;
		hairMaster.setPositionData(0, 
				(float)(pos.p[0]/scale+(headExt.currentPosition[0]+head.units[0][2]*0.08)),
				(float)(pos.p[1]/scale+(headExt.currentPosition[1]+head.units[1][2]*0.08)),
				(float)(pos.p[2]/scale+(headExt.currentPosition[2]+head.units[2][2]*0.08)));
		frontHair1 = hairArmature.getBone("FrontHair1");
		frontHair0 = hairArmature.getBone("FrontHair0");
		
		tail = new TailLink[6];
		new TailLink(pos.clone().add(1,1,32),0,tail,getDepth()-1,(int)(4*scale)/16,tailArmature,0,this).prev=this;
		for(TailLink t: tail)
			{
			t.distTolerance=0;
			t.collisionHardness=1;
			}
		hair = new TailLink[6];
		new TailLink(pos.clone().add(1,-16,48),0,hair,getDepth()-1,(int)(3*scale)/16,hairArmature,3,this).prev=this;
		
		
		//Compute frames and stuff
		tailArmature.computeFrame(0, 1, false);
		hairArmature.computeFrame(0, 1, false);
		armature.computeFrame(0, 1, false);
		
		lastHairZ = headExt.currentPosition[1];
		hairDir = 0;
		hairDirSp = 0;
		frame=0;
		animationIndex=-1;
		lastFrame=frame;
		lookAt = new V3D(160,-120,640);
		}
	public void animate(int index)
		{
		frame = PonyAnimator.FRAMES[index];
		animationIndex = index; 
		}
	
	@Override
	public void step()
		{
		
		if(frame>90 && frame<145)
			lookAt.makeAverage(pos.x()+480, pos.y(), -640,0.25);
		else if(frame>190 && frame<220)
			lookAt.makeAverage(pos.x()-160, pos.y()-3840, 320,0.25);
		else if(frame>250 && frame<280)
			lookAt.makeAverage(pos.x()+160, pos.y()-3840, 320,0.25);
		else if(frame>310 && frame<355)
			lookAt.makeAverage(pos.x(), pos.y()-3840, 320,0.25);
		else if (frame<440)
			lookAt.makeAverage(pos.x(),pos.y()-120,640,0.25);
		else if(frame>=694 && frame<739)
			{
			lookAt.makeAverage(pos.x(),pos.y()-120,640,0.25);
			talking = true; 
			talkTimer=0;
			alarm[0] = 2;
			}
		else if(flipVision)
			lookAt.makeAverage(pos.x()-320,pos.y()-64,0,0.25);
		else if (frame<760)
			lookAt.makeAverage(pos.x()+320,pos.y()-80,0,0.25);
		else if (frame>=822 && frame<906)
			lookAt.makeAverage(0,pos.y(),64,0.25);
		else
			lookAt.makeAverage(160,pos.y()-120,640,0.25);
		
		//if(animationIndex==DANCEJUMP && frame>=800)
			//blinkTimer=1;
		
		if(animationIndex==BUTTLOOP || animationIndex==DANCEJUMP)
			{
			talking = true;
			talkTimer=0;
			alarm[0] = 2;
			}
			
		blinkTimer-=1;
		if(blinkTimer<0)
			blinkTimer+=75+(float)(Math.random()*75);
		
		if(framePassed(553) && animationIndex==TURN)
			flipVision = true;
		
		if(framePassed(28) || framePassed(45) || framePassed(52))
			Audio.get("clop").play();
		if(framePassed(194) && animationIndex==POINTLEFT)
			{Audio.get("GoHit").play();
			new Stamp(new V3D(48,-240+32,0),"aKey");
			new Stamp(new V3D(48,-240+80,0),"pon");}
		if(framePassed(314) && animationIndex==POINTUP)
			{Audio.get("WheeHit").play();
			new Stamp(new V3D(160,-240+16,0),"sKey");
			new Stamp(new V3D(160,-240+64,0),"pata");}
		if(framePassed(253) && animationIndex==POINTRIGHT)
			{Audio.get("HeyHit").play();
			new Stamp(new V3D(320-48,-240+32,0),"dKey");
			new Stamp(new V3D(320-48,-240+80,0),"chaka");}
		
		
		if(animationIndex!=-1)
			{
			if(framePassed(FRAMES[animationIndex+1]))
				{
				int ind = animationIndex;
				animationIndex=-1;
				listener.ponyAnimatorAnimationComplete(ind);
				}
			}
		
		int fframe = (int)Math.min(frame, armature.frames-1);
		
		talkTimer+=1;
		if(talkTimer>=15)
			talkTimer=0;
		if(talking && talkTimer<7)
			{
			jaw.setPositionData((int)fframe, jawX+0f, jawY+0.185f, jawZ-0.109f);
			jaw.setRotationData((int)fframe, 2.622f, 0f, 0f);
			}
		else
			{
			talkTimer=0;
			jaw.setPositionData((int)fframe, jawX, jawY, jawZ);
			jaw.setRotationData((int)fframe, 0f, 0f, 0f);
			}
		

		if(interp<1)
			interp+=1f/8;
		else
			interp=1;
		
		armature.computeFrame((int) fframe,interp,mirror);
		
		lastFrame = frame;
		if(animationIndex!=-1)
			{
			if(animate)
				frame+=(animationIndex==RUN?2:1);
			}
		
		super.step();
		
		stepHairTailUpdate();
		stepHairTailCollisions();
		
		lookAt(lookAt);
		}
	
	boolean framePassed(int i)
		{
		return (frame>=i && lastFrame<i);
		}
	@Override
	public void alarm(int i)
			{
			if(i==0)
				talking = false;
			}
	
	@Override
	public void destroy()
		{
		super.destroy();
		for(TailLink t:tail)
			t.destroy();
		for(TailLink t:hair)
			t.destroy();
		}
	
	protected void stepHairTailUpdate()
		{
		double a,b;
		//HAIR MOTION
		hairMaster.setPositionData(0, 
				(float)((headExt.currentPosition[0])),
				(float)((headExt.currentPosition[1])),
				(float)((headExt.currentPosition[2])));
		hairMaster.copyPostMatrix(headExt.units);
		frontHair0.setRotationData(0, 10+hairDir/4, 0, 0);
		if(hairDirSp>1.6f) hairDirSp=1.6f; else if(hairDirSp<-1.6f) hairDirSp=-1.6f;
		hairDirSp-=hairDir/32;
		hairDirSp*=0.95;
		hairDir+=hairDirSp;
		hairDirSp+=(headExt.currentPosition[1]-lastHairZ)*8;
		lastHairZ = headExt.currentPosition[1];
		frontHair1.setRotationData(0, hairDir+5, 0, 0);
		
		hairArmature.computeFrame(0, 1, false);
				
		hair[0].updatePosition(
				pos.p[0]+(headExt.currentPosition[0]+headExt.units[0][2]*0.04)*scale,
				pos.p[1]+(headExt.currentPosition[1]+headExt.units[1][2]*0.04)*scale,
				pos.p[2]+(headExt.currentPosition[2]+headExt.units[2][2]*0.04)*scale);
		a=0.5;
		b=-0.4;
		hair[1].speed.add(
				headExt.units[0][1]*a+headExt.units[0][2]*b,
				headExt.units[1][1]*a+headExt.units[1][2]*b,
				headExt.units[2][1]*a+headExt.units[2][2]*b);
		a=0.2;
		b=-0.1;
		hair[2].speed.add(
				headExt.units[0][1]*a+headExt.units[0][2]*b,
				headExt.units[1][1]*a+headExt.units[1][2]*b,
				headExt.units[2][1]*a+headExt.units[2][2]*b);
		a=0.1;
		hair[3].speed.add(
				headExt.units[0][1]*a,
				headExt.units[1][1]*a,
				headExt.units[2][1]*a);
		a=0.43;
		b=0.14;
		//TAIL MOTION
		tail[0].updatePosition(
				pos.p[0]+(hip.currentPosition[0]+hip.units[0][1]*a+hip.units[0][2]*b)*scale,
				pos.p[1]+(hip.currentPosition[1]+hip.units[1][1]*a+hip.units[1][2]*b)*scale,
				pos.p[2]+(hip.currentPosition[2]+hip.units[2][1]*a+hip.units[2][2]*b)*scale);
		a=0.6;
		b=1.4;
		tail[1].speed.add(
				hip.units[0][1]*a+hip.units[0][2]*b,
				hip.units[1][1]*a+hip.units[1][2]*b,
				hip.units[2][1]*a+hip.units[2][2]*b);
		a=0.6;
		b=0.6;
		tail[2].speed.add(
				hip.units[0][1]*a+hip.units[0][2]*b,
				hip.units[1][1]*a+hip.units[1][2]*b,
				hip.units[2][1]*a+hip.units[2][2]*b);
		}
	protected void stepHairTailCollisions()
		{
		double r1,r2,r3,r4;
		//Hair Collisions Spheres(Head,Body,Butt,Neck)
		c1.set(
				pos.p[0]+(headExt.currentPosition[0]+headExt.units[0][1]*-0.82+headExt.units[0][2]*-0.15)*scale,
				pos.p[1]+(headExt.currentPosition[1]+headExt.units[1][1]*-0.82+headExt.units[1][2]*-0.15)*scale,
				pos.p[2]+(headExt.currentPosition[2]+headExt.units[2][1]*-0.82+headExt.units[2][2]*-0.15)*scale);
		c2.set(
				pos.p[0]+(crossBack.currentPosition[0]+crossBack.units[0][1]*-0.48+crossBack.units[0][2]*-0.3)*scale,
				pos.p[1]+(crossBack.currentPosition[1]+crossBack.units[1][1]*-0.48+crossBack.units[1][2]*-0.3)*scale,
				pos.p[2]+(crossBack.currentPosition[2]+crossBack.units[2][1]*-0.48+crossBack.units[2][2]*-0.3)*scale);
		c3.set(
				pos.p[0]+(hip.currentPosition[0]+hip.units[0][2]*-0.3)*scale,
				pos.p[1]+(hip.currentPosition[1]+hip.units[1][2]*-0.3)*scale,
				pos.p[2]+(hip.currentPosition[2]+hip.units[2][2]*-0.3)*scale);
		c4.set(
				pos.p[0]+(crossBack.currentPosition[0]+crossBack.units[0][1]*-0.7+crossBack.units[0][2]*0.18)*scale,
				pos.p[1]+(crossBack.currentPosition[1]+crossBack.units[1][1]*-0.7+crossBack.units[1][2]*0.18)*scale,
				pos.p[2]+(crossBack.currentPosition[2]+crossBack.units[2][1]*-0.7+crossBack.units[2][2]*0.18)*scale);
		r1 = 16*1.127*scale/16;
		r2 = 10*scale/16;
		r3 = 11*scale/16;
		r4 = 9*scale/16;
		for(int i=1; i<hair.length; i++)
			{
			hair[i].collisionBall(c1,r1,speed,0.5,false);
			hair[i].collisionBall(c2,r2,speed,0.5,false);
			hair[i].collisionBall(c3,r3,speed,0.5,false);
			hair[i].collisionBall(c4,r4,speed,0.5,false);
			}
		//Tail Collisions Spheres(butt,left leg, right leg,body)
		c1.set(
				pos.p[0]+(hip.currentPosition[0]+hip.units[0][2]*-0.75)*scale,
				pos.p[1]+(hip.currentPosition[1]+hip.units[1][2]*-0.75)*scale,
				pos.p[2]+(hip.currentPosition[2]+hip.units[2][2]*-0.75)*scale);
		c2.set(
				pos.p[0]+(footL.currentPosition[0])*scale,
				pos.p[1]+(footL.currentPosition[1])*scale,
				pos.p[2]+(footL.currentPosition[2])*scale);
		c3.set(
				pos.p[0]+(footR.currentPosition[0])*scale,
				pos.p[1]+(footR.currentPosition[1])*scale,
				pos.p[2]+(footR.currentPosition[2])*scale);
		c4.set(
				pos.p[0]+(crossBack.currentPosition[0]+crossBack.units[0][1]*-0.48+crossBack.units[0][2]*-0.3)*scale,
				pos.p[1]+(crossBack.currentPosition[1]+crossBack.units[1][1]*-0.48+crossBack.units[1][2]*-0.3)*scale,
				pos.p[2]+(crossBack.currentPosition[2]+crossBack.units[2][1]*-0.48+crossBack.units[2][2]*-0.3)*scale);
		r1 = 12*scale/16;
		r2 = 6*scale/16;
		r3 = 6*scale/16;
		r4 = 10*scale/16;
		for(int i=1; i<tail.length; i++)
			{
			tail[i].collisionBall(c1,r1,speed,0.5,false);
			tail[i].collisionBall(c2,r2,speed,0.5,false);
			tail[i].collisionBall(c3,r3,speed,0.5,false);
			tail[i].collisionBall(c4,r4,speed,0.5,false);
			}
		}
	
	/**Create eye angle vectors. Does not rotate head.*/
	protected void lookAt(V3D target)
		{
		V3D headUp=new V3D(),diffVector=new V3D(),flat=new V3D();
		double d,diff,diff2;
		headUp.set(head.units[0][2],head.units[1][2],head.units[2][2]).normalize();
		diffVector.set(target).add(-head.currentPosition[0]*scale-pos.p[0],-head.currentPosition[1]*scale-pos.p[1],-head.currentPosition[2]*scale-pos.p[2]);
		d = headUp.dot(diffVector);
		flat.set(target).add(headUp,-d).add(-head.currentPosition[0]*scale-pos.p[0],-head.currentPosition[1]*scale-pos.p[1],-head.currentPosition[2]*scale-pos.p[2]);
		diff  = new V3D(-head.units[0][1],-head.units[1][1],-head.units[2][1]).angle(flat)*
				Math.signum(new V3D(-head.units[0][0],-head.units[1][0],-head.units[2][0]).dot(flat));
		
		diff2 = new V3D(-head.units[0][2],-head.units[1][2],-head.units[2][2]).angle(diffVector);
		diff = Calc.toDeg(diff);
		diff2 = Calc.toDeg(diff2)-90;
		
		eyeAngleZL = (float) (36.7 + 0.2755d*diff + 0.000478d*Math.pow(diff,2) - 0.000001856d*Math.pow(diff,3));
		eyeAngleZR = -(float) (36.7 + 0.2755d*-diff + 0.000478d*Math.pow(-diff,2) - 0.000001856d*Math.pow(-diff,3));
		eyeAngleXL = (float) (90 - 0.1852d*diff2 - 0.00001828d*Math.pow(diff2,3));
		eyeAngleXR = eyeAngleXL;
		}
	
	
	@Override
	public void render()
		{
		model.updateToArmature();
		if(blinkModel!=null)
			blinkModel.updateToArmature();
		if(hairModel!=null)
			hairModel.updateToArmature();
		if(tailModel!=null)
			tailModel.updateToArmature();
		
		GL2 gl = GFX.gl;
		gl.glColor3f(1,1,1);

		gl.glPushMatrix();
		
		gl.glTranslated(pos.x(), pos.y(), pos.z());
		gl.glScaled(scale ,scale,scale);
		
		gl.glEnable(GL2.GL_LIGHTING);
		gl.glEnable(GL.GL_CULL_FACE);
		gl.glCullFace(GL.GL_BACK);
		gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDepthFunc(GL.GL_LEQUAL);
        gl.glHint(GL2.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
        gl.glClearDepth(1.0f);
        gl.glClear(GL2.GL_DEPTH_BUFFER_BIT);
        
		renderModels(true);
		gl.glPopMatrix();

		gl.glDisable(GL.GL_CULL_FACE);
		gl.glDisable(GL.GL_DEPTH_TEST);
		gl.glDisable(GL2.GL_LIGHTING);
		
		gl.glColor3f(1,1,1);
		}
	
	protected void renderModels(boolean useMaterials)
		{
		GL2 gl = GFX.gl;
		model.useMaterials=useMaterials;
		if(blinkModel!=null)
			blinkModel.useMaterials=useMaterials;
		if(pupilModel!=null)
			pupilModel.useMaterials=useMaterials;
		if(hairModel!=null)
			hairModel.useMaterials=useMaterials;
		if(tailModel!=null)
			tailModel.useMaterials=useMaterials;
		/**Render model, wings*/
		
		model.render();
		if(hairModel!=null)
			hairModel.render();
		if(tailModel!=null)
			tailModel.render();
		
		if(blinkModel!=null && blinkTimer<3)
			blinkModel.render();
		if(blinkTimer>=3 && pupilModel!=null)
			{
			gl.glPushMatrix();
			head.transformTo(new float[]{0.0f,-0.8f*1.017f,2.490f*1.017f});
			/**Render pupils*/
			gl.glPushMatrix();
			gl.glRotatef(eyeAngleZL,0,0,-1);
			gl.glRotatef(eyeAngleXL,1,0,0);
			gl.glScalef(128*0.0017f,128*0.0022f,128*0.00155f);
			pupilModel.render();
			gl.glPopMatrix();
			
			gl.glPushMatrix();
			gl.glRotatef(eyeAngleZR,0,0,-1);
			gl.glRotatef(eyeAngleXR,1,0,0);
			gl.glScalef(-128*0.0017f,128*0.0022f,128*0.00155f);
			GFX.swapCullFace();
			pupilModel.render();
			GFX.swapCullFace();
			gl.glPopMatrix();
			
			/**Render eyeball*/
			if(useMaterials)
				{gl.glScalef(0.417f, 0.417f,-0.417f);//inverted to achieve culling
				MTL.getMaterial("pinkie.mtl", "eye_dash.png").enable(0);}
			else
				gl.glScalef(0.447f, 0.447f, 0.447f);//larger
			GFX.unitSphere2.render();
			if(useMaterials)
				MTL.getMaterial("pinkie.mtl", "eye_dash.png").disable();
			
			gl.glPopMatrix();
			}
		}
	public void setAnimationListener(AnimationListener listener)
		{
		this.listener = listener;
		}
	}
