package game;

import com.jogamp.opengl.GL2;


import graphics.Armature;
import graphics.Bone;
import graphics.GFX;
import calc.Calc;
import calc.V3D;

public class TailLink extends SphereEntity
	{
	public SphereEntity prev,next,parentObject;
	public int tailIndex;
	public Bone tailBone;
	public double collisionHardness;
	/**Workspace*/	
	public V3D c1,c2;
	
	double maxDist;
	
	public TailLink(V3D pos, int index, TailLink[] tail, int depth, int maxDist, Armature tailArmature, int boneIndex,PonyAnimator ponyAnimator)
		{
		super(pos);
		if(index==0)
			gravity = V3D.ZERO;
		else
			gravity = gravity.multiply((double)ponyAnimator.radius/16);
		c0 = new V3D();
		c1 = new V3D();
		c2 = new V3D();
		visible=false;
		radius=1;
		distTolerance=0;
		collisionHardness = 1;
		frict=0.0;
		setDepth(depth);
		prev=null;
		next=null;
		this.parentObject=ponyAnimator;
		this.maxDist = maxDist;
		if(boneIndex<tailArmature.bones.length)
			tailBone = tailArmature.bones[boneIndex];
		else
			tailBone = null;
		tailIndex=index;
		tail[index] = this;
		if(tailIndex+1<tail.length)
			{
			next = new TailLink(pos.clone().add(0, -1, 0),index+1,tail,depth-1,maxDist,tailArmature,boneIndex+1,ponyAnimator);
			((TailLink)next).prev=this;
			}
		}
	
	@Override
	public void step()
		{
		super.step();
		stringPhysics();
		updateBone();
		}
	public void stringPhysics()
		{
		if(prev!=null && prev instanceof TailLink)
			{
			speed.accelerate((-Math.pow(speed.len(), 2)/512));
			if(((TailLink)prev).prev==null)//Previous is root
				{
				c0.set(pos).add(prev.pos, -1);
				V3D diff = c0;
				//double dist = diff.len();
				diff.normalize();
				c1.set(speed).add(prev.speed,-1);
				V3D diffSpeed = c1;
				double d = diffSpeed.dot(diff);
				if(diff.len()>=maxDist)
					{
					pos.set(prev.pos).add(diff,maxDist);
					speed.add(diff,-d);
					}
				}
			else
				{
				speed.makeAverage(prev.y,0.05);
				c0.set(pos).add(prev.pos, -1);
				V3D diff = c0;
				double dist = diff.len();
				diff.normalize();
				c1.set(speed).add(prev.speed,-1);
				V3D diffSpeed = c1;
				double d = diffSpeed.dot(diff);
				if(dist>maxDist)
					{
					pos.makeAverage(c2.set(prev.pos).add(diff,maxDist),0.95);
					prev.pos.makeAverage(c2.set(pos).add(diff,-maxDist),0.95);
					prev.speed.add(diff,d*0.95);
					speed.add(diff,-d*0.95);
					speed.accelerate(-0.01*speed.sqLen());
					}
				}
			}
		}
	public void updateBone()
		{
		if(tailBone!=null)
			{
			tailBone.currentPosition = c0.set(pos).sub(parentObject.pos).multiply(1d/parentObject.scale).toFloat();
			y.makeAverage(c0.set(next.pos).add(pos,-1).normalize(),0.5);
			if(prev==null || !(prev instanceof TailLink))
				z.makeAverage(parentObject.z,0.5);
			else
				z.makeAverage(c0.set(prev.z).add(prev.y,0.25).add(parentObject.z,2/tailIndex),0.25);
			x.setCross(z, y);
			x.normalize();
			z.setCross(y, x);
			z.normalize();
			
			
			tailBone.units[0] = x.toFloat();
			tailBone.units[1] = y.toFloat();
			tailBone.units[2] = z.toFloat();
			tailBone.units = Calc.transpose3(tailBone.units);
			}
		}

	@Override
	public void render()
		{
		GL2 gl = GFX.gl;
		gl.glPushMatrix();
		gl.glTranslated(pos.x(),pos.y(),pos.z());
		gl.glScalef(16, 16, 16);
		GFX.unitSphere1.render();
		gl.glPopMatrix();
		if(next!=null)
			{
			gl.glBegin(GL2.GL_LINE_STRIP);
			gl.glVertex3d(pos.x(),pos.y(),pos.z());
			gl.glVertex3d(next.pos.x(),next.pos.y(),next.pos.z());
			gl.glEnd();
			}
		}
	@Override
	public void collide(V3D referencePoint, V3D referenceSpeed,V3D normal,double referenceRadius, double friction)	
		{
		//Simplified. Does not set unit vectors!
		c0.set(speed).add(referenceSpeed,-1);
		V3D speedDiff = c0;
		double d = -normal.dot(speedDiff);
		pos.makeAverage(c1.set(referencePoint).add(normal,referenceRadius+radius),collisionHardness);
		if(d>0)
			{
			speed.add(normal,d);
			//speed.accelerate(-d*friction*frict*Main.time);
			}
		}
	
	public void updatePosition(double x, double y, double z)
		{
		speed.x(x-pos.x());
		speed.y(y-pos.y());
		speed.z(z-pos.z());
		pos.x(x-speed.p[0]);
		pos.y(y-speed.p[1]);
		pos.z(z-speed.p[2]);
		}
	}
