package main;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashSet;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import com.jogamp.nativewindow.AbstractGraphicsDevice;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.awt.GLCanvas;
import com.jogamp.opengl.util.Animator;
import com.jogamp.opengl.util.AnimatorBase;
import mechanics.Scene;

import graphics.GFX;
import graphics.MTL;
import io.IOH;

import audio.*;

/**
 * This class sets up what we need in general.
 */

public class Main
{
	public static BufferedImage icon;

	static int preferredWidth =	640;
	static int preferredHeight=	480;
	private static int originalWidth;
	private static int originalHeight;
	public  static boolean ignoreResize = false;
	public  static int FPS=30;
	public  static boolean isFullscreen=false;
	private static final String TITLE="Pika pika";

	private static Integer x = null;
	private static Integer y = null;
	public static double airResistance;
	public static final double GROUND_FRICTION_STANDARD = 0.6;

	public static Cursor noCursor;
	public static Frame frame;
	public static JFrame loadFrame;
	public static BufferedImage loadImage,screenImage;
	protected static double loadProgress,targetLoadProgress;

	public static IOH io; 
	public static Scene scene;
	public static AnimatorBase animator;
	public static AbstractGraphicsDevice device;
	public static GLProfile profile;
	public static GLCapabilities caps;
	public static GLCanvas canvas;
	public static Audio audio;
	public static GFX gfx;
	public static MTL mtl;
	public static int audioBuffer = 8;
	public static int sampleRate = 22050;

	public static boolean alwaysUseDisplayLists = false;
	public static boolean forcePOTs = false;
	public static boolean useVBO = true;
	public static boolean bilinearMag = false;
	public static boolean bilinearMin = false;

	public static void main(String[] args)
	{
		try {
			icon = ImageIO.read(new File("res/icon.png"));
		} catch(IOException err) {
			err.printStackTrace();
		}

		final HashSet<String> argSet = new HashSet<String>();

		for(int i = 0; i<args.length; i++)
		{
			int s = 0;
			while(s<args[i].length() && args[i].charAt(s) == '-')
				s++;
			argSet.add(args[i].substring(s));
		}

		if(argSet.contains("skipSettings"))
			new Main(argSet);
		else
		{
			SwingUtilities.invokeLater(new Runnable()
			{
				@Override
				public void run()
				{
					new SettingsConfig(argSet);
				}
			});
		}
	}

	public Main(HashSet<String> args)
	{
		loadProgress = 0;
		setTargetLoadProgress(0.5);
		if(!args.contains("noErrorHandler"))
			System.setErr(new ErrorHandler().getStream());

		SwingUtilities.invokeLater(new Runnable()
		{
			@Override
			public void run()
			{
				showLoaderFrame();
				Timer t = new Timer(100, new ActionListener()
				{
					@Override
					public void actionPerformed(ActionEvent e)
					{
						if(loadFrame == null)
							((Timer)e.getSource()).stop();
						else
						{
							loadProgress = (loadProgress*5+getTargetLoadProgress())/6;
							loadFrame.getContentPane().repaint();
							loadFrame.repaint();
						}
					}
				});
				t.start();
			}
		});

		//GLProfile.initSingleton();
		setTargetLoadProgress(0.75);
		Settings.readSettings();
		Settings.writeSettings();

		airResistance=0.0007;

		// create a blank cursor
		Toolkit t = Toolkit.getDefaultToolkit();
		Image i = new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
		noCursor = t.createCustomCursor(i, new Point(0, 0), "none"); 

		audio = new Audio();
		setTargetLoadProgress(1);
		System.out.println("Initializing OpenGL");
		device = GLProfile.getDefaultDevice();
		profile = GLProfile.get(device,GLProfile.GL2);
		caps = new GLCapabilities(profile);
		caps.setAlphaBits(0);
		caps.setRedBits(8);
		caps.setGreenBits(8);
		caps.setBlueBits(8);
		caps.setBackgroundOpaque(true);
		//caps.setDepthBits(depthBits)
		caps.setDoubleBuffered(true);
		caps.setHardwareAccelerated(true);
		caps.setSampleBuffers(false);
		caps.setStencilBits(0);
		caps.setStereo(false);

		scene = new Scene(preferredWidth,preferredHeight);//Note: scene instantiates the GFX
		System.out.println("Creating GLCanvas.");
		if(canvas != null)
			throw new RuntimeException("Canvas already exists!");
		canvas = new GLCanvas(caps);
		canvas.addGLEventListener(scene);
		canvas.setPreferredSize(new Dimension(preferredWidth,preferredHeight));
		io = new IOH(this);

		//This is the most random workaround ever
		//But it increases the timing resolution on windows.
		Thread sleeper = new Thread(){
			@Override public void run(){
				try {sleep(Long.MAX_VALUE);}
				catch (InterruptedException e){}
			}
		};
		sleeper.setDaemon(true);
		sleeper.start();

		//animator = new FPSAnimator(canvas, FPS, scheduleAtFixedRate);
		animator = new Animator();
		animator.add(canvas);
		((Animator)animator).setRunAsFastAsPossible(true);

		frame = new Frame(TITLE)
		{
			@Override
			public void dispose()
			{
				System.out.println("System exit");
				animator.stop();
				scene.dispose(null);//obs
				super.dispose();
				System.exit(0);
			}
		};
		if(Main.icon != null)
			frame.setIconImage(Main.icon);
		frame.setCursor(noCursor);
		frame.setUndecorated(true);
		frame.add(canvas);
		frame.setResizable(false);
		frame.pack();
		originalWidth=frame.getWidth();
		originalHeight=frame.getHeight();

		frame.addWindowListener(new WindowAdapter()
		{@Override
			public void windowClosing(WindowEvent e)
		{frame.dispose();}
		});



		frame.addComponentListener(new ComponentAdapter()
		{@Override
			public void componentResized(ComponentEvent e)
		{if (!ignoreResize)
		{originalWidth=e.getComponent().getWidth();
		originalHeight=e.getComponent().getHeight();}
		ignoreResize=false;}
		});
		canvas.requestFocusInWindow();

		//sleep(300);
		System.out.println("Attempting to display GLCanvas.");
		frame.setVisible(true);
		//canvas.display();
	}
	private synchronized static void setTargetLoadProgress(double d)
	{
		targetLoadProgress = d;
	}
	private synchronized static double getTargetLoadProgress()
	{
		return targetLoadProgress;
	}

	private void showLoaderFrame()
	{
		try
		{
			loadFrame = new JFrame();
			if(Main.icon != null)
				loadFrame.setIconImage(Main.icon);
			loadFrame.setUndecorated(true);
			File img = new File("res/in.png");
			if(!img.exists())
			{
				error("The loader image \"in.png\" was missing.\n" +
						"Please make sure you have extracted the\n" +
						"archive to your disk before running the game.\n" +
						"\n" +
						"Exiting game.", "Loader image missing");
				throw new IOException("Missing image file");
			}

			loadImage = ImageIO.read(img);
			JPanel p = new JPanel(){
				@Override
				public void paintComponent(Graphics g)
				{g.drawImage(screenImage,0,0,null);
				g.drawImage(loadImage,(int)(260*(1-loadProgress)),0,260,410,0,0,(int)(260*(loadProgress)), 410, null);
				g.drawImage(loadImage,0,0,300,410,300,0,600, 410, null);}
			};
			p.setPreferredSize(new Dimension(260,410));
			loadFrame.setContentPane(p);
			loadFrame.pack();
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			loadFrame.setLocation(screenSize.width-260, screenSize.height/2-205);
			screenImage = IOH.robot.createScreenCapture(loadFrame.getBounds());
			loadFrame.setVisible(true);
		}
		catch (IOException e)
		{System.err.println("Loader Image Missing");
		System.exit(1);}
	}

	/**This will show a JOptionPane error message.*/
	public static void error(String message, String caption)
	{
		JOptionPane.showMessageDialog(null,message,caption,JOptionPane.ERROR_MESSAGE);
	}

	public static void setFullScreen(boolean full)
	{
		System.out.println(full+", "+originalWidth+", "+originalHeight);
		isFullscreen = full;
		SwingUtilities.invokeLater(new Runnable()
		{
			@Override
			public void run()
			{
				if(isFullscreen)
				{
					ignoreResize=true;
					Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
					frame.setBounds(0,0,screenSize.width, screenSize.height);
				}
				else
					frame.setBounds(getXPos(),getYPos(),originalWidth, originalHeight);
			}
		});
	}
	private static int getXPos()
	{
		if(x==null)
		{
			Rectangle bb = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().getBounds();
			return (int) (bb.getCenterX()-originalWidth/2);
		}
		else
			return x;
	}
	private static int getYPos()
	{
		if(y==null)
		{
			Rectangle bb = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().getBounds();
			return (int) (bb.getCenterY()-originalHeight/2);
		}
		else
			return y;
	}
	/**Sleep without exceptions.*/
	public static void sleep(int i)
	{
		try {Thread.sleep(i);}
		catch (InterruptedException e){}
	}
}